#!/usr/bin/python3

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtPrintSupport import *

from shutil import rmtree
import subprocess, os, sys, time
from pathlib import Path
import psycopg2

QTEXT = 0	# indices for the rows from the database
QATEXT = 1
QAUTHOR = 2
QPART = 3
QTYPE = 4
QSN = 5
QRELATIONS = 6
QMARK = 7
QSTAT = 8
QFIGNAME = 9
QFIG = 10

#offsets of the fields in the file
FSTAT = 0
FAUTH = 1
FPN = 2
FTYPE = 3
FREL = 4
FMARK = 5
FQTEXT = 6
FATEXT = 7

NFRAGS = 8		# text file contains N sections separated by +++

class Course:
	def __init__(self):
		self.password = None
		self.remoteCourseCode = None			# will be set by openDatabase()
		self.language = 'english'
		self.courseCode = None
		self.projectDir = None
		self.courseDir = None
		self.path = None						# default file name
		self.author = ''
		self.partnum = None
		self.qtype = None
		self.relations = ''
		self.mark = None
		self.questionLocked = False

	def selectRemoteCourse(self, code):  # select from the Course
		pw = GUI.getPassword(code)	
		if pw == None:
			GUI.statusBar().showMessage('No password obtained for %s.'%code)	
			return
		if 1:
			print (GUI.db, GUI.host, code.lower(), pw)
			conn = psycopg2.connect(database=GUI.db, host=GUI.host,
				user=code.lower(), password=pw)				
			cursor = conn.cursor()	
			cursor.execute("SELECT count(*) FROM %s"%code)	
			res = cursor.fetchone()
		else:
			GUI.dialog_critical('Connection to database %s failed.'%code)
			return
		GUI.statusBar().showMessage('Selected %s.Found %d rows'%(code,res[0]))
		self.coursepassword = pw		# Use later for import / export
		self.remoteCourseCode = code
		self.language = 'english'
		for c in GUI.courseList:
			if c[0] == code:
				self.courseName = c[1]
				self.subject = c[2]
				self.degree = c[3]
				self.syllabusYear = c[4]
				self.semester = c[5]
				self.language = c[6]
				self.qpModel = c[7]
				self.duration = c[8]

	def importCourse(self):	
		if self.projectDir == None:
			GUI.dialog_critical('Set the Project Directory first.')
			return	

		if self.remoteCourseCode == None:
			GUI.dialog_critical('No Course selected from Database yet.')
			return	
		
		cdir = os.path.join(self.projectDir, self.remoteCourseCode)
		
		if os.path.isdir(cdir) == False:
			msg = 'Local Project Directory %s does not have %s. Do you want to create it?'%(self.projectDir, self.remoteCourseCode)
			reply = QMessageBox.question(GUI, 'Create Course Directory', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
			if reply == QMessageBox.No:
				return
		else:
			msg = 'Directory %s exists, contents will be deleted. Do you want to proceed?'%cdir
			reply = QMessageBox.question(GUI, 'Create Course Directory', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
			if reply == QMessageBox.No:
				return
			rmtree(cdir)
		
		os.chdir(self.projectDir)				
		os.mkdir(self.remoteCourseCode) 
		os.chdir(self.remoteCourseCode) 
		for k in range(1,9):
			os.mkdir('P%d'%k)
		self.courseDir = cdir
		os.chdir(self.courseDir)
		self.courseCode = self.remoteCourseCode				
		GUI.statusBar().showMessage('Current Course Directory is %s'%cdir)
		GUI.courseDirButton.setText('Course Directory : %s'%self.courseCode)		
		
		pf = open('properties', 'w')
		pf.write('courseName:%s\n'%self.courseName)
		pf.write('subject:%s\n'%self.subject)
		pf.write('degree:%s\n'%self.degree)
		pf.write('syllabusyear:%s\n'%self.syllabusYear)
		pf.write('semester:%s\n'%self.semester)
		pf.write('language:%s\n'%self.language)
		pf.write('qpModel:%s\n'%self.qpModel)
		pf.write('duration:%s\n'%self.duration)
		pf.close()

		db = GUI.db
		host = GUI.host
		print (db, host, self.coursepassword)		
		if 1:
			conn = psycopg2.connect(database=db, host=host, user=self.remoteCourseCode.lower(), password=self.coursepassword)
			cursor = conn.cursor()	
			cursor.execute("SELECT * FROM %s"%self.remoteCourseCode)	
			results = cursor.fetchall()		
		else:
			GUI.dialog_critical('Import from Database %s failed.'%self.remoteCourseCode)
		
		nr = len(results)
		for k in range(nr):				# Save the results to the Course directory
			row = results[k]
			part = row[QPART]      
			os.chdir('P%d'%part)		# Change to folder Pn
			if row[QFIGNAME] != None:
				open(row[QFIGNAME], 'wb').write(row[QFIG])	
			else:
				print ('status = ', row[QSTAT])		
				if row[QSTAT] == 'inactive':
					matter = 'status:inactive\n+++\n'
				else:
					matter = 'status:active\n+++\n'	
				matter = 'status:%s\n+++\n'%row[QSTAT]
					
				matter += 'Author:%s\n+++\nPart Number:%d\n+++\nType:%s\n+++\n'%(row[QAUTHOR],row[QPART],row[QTYPE])
				matter += 'Relations:%s\n+++\nmark:%d\n+++\n%s\n+++\n%s\n'%(row[QRELATIONS],row[QMARK], row[QTEXT], row[QATEXT])
				fn = 'P%d-%s%02d.txt'%(row[QPART], row[QTYPE], row[QSN])
				print(fn)
				f = open(fn,'w')
				f.write(matter)
				f.close()
			os.chdir('..')	
		count = 0
		for row in results:
			if row[QTEXT] != None:
				count += 1	
		GUI.statusBar().showMessage('Downloaded %d Records'%count)	
		GUI.update_title('')		
		GUI.authorEdit.setText('')
		GUI.partNumberEdit.setText('')
		GUI.questionTypeEdit.setText('')
		GUI.relationsEdit.setText('')
		GUI.markEdit.setText('')
		GUI.qeditor.setPlainText('')
		GUI.aeditor.setPlainText('')
		self.path = cdir
		# end importCourse()


	def checkFiles(self):				# verify the format of files
		if self.courseDir == None:
			GUI.dialog_critical('Select a Course Diectory first.')
			return	
		count = 0
		os.chdir(self.courseDir)
		for part in range(1,9):		# traverse directories P1 to P8
			wd = str(Path.cwd()) + '/' + 'P%d'%part
			os.chdir(wd)	# change directory to the desired Part
			files = Path('.').glob('*.txt')
			ana = []		# store question numbers for error checking
			bna = []
			cna = []
			for fn in files:
				fn = str(fn)
				f = open(fn)
				filetext = f.read()
				content = filetext.split('+++')	
				f.close()
				if len(fn) != 10:
					GUI.dialog_critical('Part %d: %s .Filename format is wrong.'%(part, fn))
					os.chdir('..')	
					return
					
				p = '%d'%part
				if fn[1] != p:
					GUI.dialog_critical('%s should not be inside P%d'%(fn,part))
					os.chdir('..')	
					return
									
				if len(content) != NFRAGS:
					GUI.dialog_critical('part %d: File %s should have eight "+++" separated fields'%(part,fn))
					return
					
				try:
					partnum = eval(content[FPN].split(':')[1])
					if partnum != part:
						GUI.dialog_critical('Error in %s : part %d inside P%d'%(fn, partnum,part))
				except:
					GUI.dialog_critical('%s :Part number parse error'%fn)
					return
				
				try:
					qns = fn[4:6]
					if qns[0] == '0':
						qns = qns.replace('0', '')
					qnum = eval(qns)
				except:
					GUI.dialog_critical('%s: Question number parse error'%fn)
					return

				try:
					qtype = content[FTYPE].split(':')[1][:-1]
					if qtype == 'A':
						ana.append(qnum)
					elif qtype == 'B':
						bna.append(qnum)
					elif qtype == 'C':
						cna.append(qnum)
					else:
						GUI.dialog_critical('%s : Invalid question type'%fn)
						return								
				except:
					GUI.dialog_critical('%s : Question type parse error'%fn)
					return		

				try:
					mark  = eval(content[FMARK].split(':')[1])
					if mark < 2 or mark > 13:
						GUI.dialog_critical('%s : Invalid mark'%fn)
						return
				except:
					GUI.dialog_critical('%s : Mark parse error'%fn)
					return

				author = content[FAUTH].split(':')[1]
				if len(author) < 3:
					GUI.dialog_critical('%s : Author info too short'%fn)
					return
				
				relations = content[FREL].split(':')[1].strip()
				if relations != '':
					print('%d%s(%s)'%(qnum,qtype,relations))
					if 1:
						rels = relations.split(',')
						nr = len(rels)
						relations = '' 		# make it afresh
						for k in range(nr):
							r = rels[k].strip()
							if r == '':
								continue
							rt = r[0].upper()
							if rt not in ['A', 'B', 'C']:
								GUI.dialog_critical('Relation must be A B or C.')
								return			
							rv = r[1:]
							if rv[0] == '0':		# removes one leading zero
								rv = rv[1:]
							rval = eval(rv)
							rfn = 'P%d-%s%02d.txt'%(part,rt,rval)
							if os.path.exists(rfn) == False:
								GUI.dialog_critical('Related Question of "%s" "%s" not found'%(fn, rfn))
								return
							relations += '%s%d,'%(rt,rval)
						relations = relations[:-1]
					else:
						GUI.dialog_critical('Improper Relationship entry')
						return								
				
				qtext = content[FQTEXT].strip()
				if len(qtext) < 3:
					GUI.dialog_critical('%s : Question too short'%fn)
					return
				
				#if "'" in qtext:print('Question contains single quotes', fn)

				atext = content[FATEXT].strip()

				#if "'" in atext: print('Answer contains single quotes', fn)

				fragments = qtext.split('\\includegraphics')
				nfig = len(fragments)
				for k in range(1,nfig):
					s1 = fragments[k]
					s2 = s1.split('{')[1]
					fig = s2.split('}')[0]
					if os.path.isfile(fig) == False:
						GUI.dialog_critical('%s : include file %s NOT found'%(fn,fig))
				count += 1		# End of loop

			for k in range(1, len(ana) + 1):
				fn = 'P%d-A%02d.txt'%(part,k)
				if os.path.exists(fn) == False:
					GUI.dialog_critical('Part %d: Question "%s" not found'%(part,fn))
					return
					
			for k in range(1, len(bna) + 1):
				fn = 'P%d-B%02d.txt'%(part,k)
				if os.path.exists(fn) == False:
					GUI.dialog_critical('Part %d: Question "%s" not found'%(part,fn))
					return

			for k in range(1, len(cna) + 1):
				fn = 'P%d-C%02d.txt'%(part,k)
				if os.path.exists(fn) == False:
					GUI.dialog_critical('Part %d: Question "%s" not found'%(part,fn))
					return		
			os.chdir('..')		# End of loop
		GUI.dialog_critical('Verified %d Files'%count)

		
	def exportCourse(self):
		if self.remoteCourseCode == None:
			GUI.dialog_critical('Select a remote Course first.')
			return	
		wd = os.getcwd()	
		target = self.courseCode

		db = GUI.db
		host = GUI.host
		pw = self.coursepassword
		print (db, host, pw)		
		
		code = self.courseCode.lower()
		conn = psycopg2.connect(database=db, host=host, user=code, password=pw)
		cursor = conn.cursor()
		cursor.execute("DELETE from %s"%code)
		conn.commit()

		courseFolder = self.projectDir + '/' + self.courseCode   # Course Directory
		os.chdir(courseFolder)		# change directory to the desired Course code
		count = 0
		for part in range(1,9):		# traverse directories P1 to P8
			wd = str(Path.cwd()) + '/' + 'P%d'%part
			os.chdir(wd)	# change directory to the desired Part
			files = Path('.').glob('*.txt')
			for fn in files:
				f = open(fn)
				content = f.read().split('+++')
				f.close()
				print(str(fn))
				qns = str(fn)[4:6]
				if qns[0] == '0':
					qns = qns.replace('0', '')
				qnum = eval(qns)
				status = content[FSTAT].split(':')[1].strip()
				author = content[FAUTH].split(':')[1].strip()
				partnum = eval(content[FPN].split(':')[1])
				qtype = content[FTYPE].split(':')[1].strip()
				relations = content[FREL].split(':')[1].strip()
				try:
					mark  = eval(content[FMARK].split(':')[1])
				except:
					GUI.dialog_critical('Invalid mark %s'%(content[FMARK]))
					return
				qtext = content[FQTEXT].strip()
				qtext = qtext.replace("'","''")		# Takes care of single quotes in SQL query
				atext = content[FATEXT].strip()
				#print ('status = ', status)
				sql = "insert into %s (qtext,atext,author,partnum,qtype,qnum,relations,mark,status) values ('%s', '%s', '%s', %d, '%s', %d, '%s', %d, '%s');"%(target,qtext,atext,author,partnum,qtype,qnum,relations,mark,status)
				#print (sql)
				try:
					cursor.execute(sql)	
					count += 1						# Question record count
				except:
					GUI.dialog_critical('SQL insert failed after %d Records'%count)
					print(sql)
					os.chdir(wd)
					return
				# load image if any
				tt = qtext + atext
				fragments = tt.split('\\includegraphics')
				nfig = len(fragments)
				for k in range(1,nfig):
					s1 = fragments[k]
					s2 = s1.split('{')[1]
					fig = s2.split('}')[0]
					print(s1, s2, fig)
					try:
						drawing = open(fig, 'rb').read()
						img = psycopg2.Binary(drawing)
						sql = "insert into %s(partnum,figname, figure) values (%d,'%s', %s);"%(target,partnum,fig, img)
						cursor.execute(sql)
					except:
						GUI.dialog_critical('%s : Insert "%s" failed.'%(fn,fig))
						print(sql)
						os.chdir(wd)
						return
			os.chdir('..')		
		os.chdir(wd)
		conn.commit()
		GUI.statusBar().showMessage('Uploaded %d Records'%count)
		#End of exportCourse() 

	def generatePartTex(self,folder):		# Generates LaTex document for one Part	
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'		
		
		tex += '\\begin{document} \\noindent'
		
		mask = folder[-3:-1] + '-A*.txt'
		print(folder, mask)
		files = sorted(Path(folder).glob(mask))
		print (len(files), files)
		tex += '\\par\n{Part %s Questions }\n'%mask[1:2]
		tex += '\\hrule \\vspace{0.05in}\n \\par {Section A}\n'

		if len(files) != 0:
			tex += '\\begin{questions}\n\\pointsdroppedatright\n'	
			for fn in files:		
				name = os.path.splitext(fn)[0]
				f = open(name+'.txt')
				content = f.read().split('+++')
				f.close()
				print (name, content)
				mark  = eval(content[FMARK].split(':')[1])	
				relations = content[FREL].split(':')[1][:-1]
				tt = content[FQTEXT] + '\n\\textbf{Answer:}\n' + content[FATEXT]
				if relations == '':
					tex += '\\question[%d] %s \\droppoints\n'%(mark,tt)
				else:
					tex += '\\question[%d](%s) %s \\droppoints\n'%(mark,relations,tt)			
			tex += '\\end{questions}\n'

		tex += '\\hrule \\vspace{0.05in}\n \\par {Section B}\n'
		
		mask = folder[-3:-1] + '-B*.txt'
		files = sorted(Path(folder).glob(mask))

		if len(files) != 0:
			tex += '\\begin{questions}\n'
			for fn in files:		
				name = os.path.splitext(fn)[0]
				f = open(name+'.txt')
				content = f.read().split('+++')
				f.close()
				mark  = eval(content[FMARK].split(':')[1])	
				relations = content[FREL].split(':')[1][:-1]
				tt = content[FQTEXT] + '\n\\textbf{Answer:}\n' + content[FATEXT]
				if relations == '':
					tex += '\\question[%d] %s \\droppoints\n'%(mark,tt)
				else:
					tex += '\\question[%d](%s) %s \\droppoints\n'%(mark,relations,tt)
			tex += '\\end{questions}\n'
		
		tex += '\\hrule \\vspace{0.05in}\n \\par {Section C}\n'
		mask = folder[-3:-1] + '-C*.txt'	
		files = sorted(Path(folder).glob(mask))
		
		if len(files) != 0:
			tex += '\\begin{questions}\n'
			for fn in files:		
				name = os.path.splitext(fn)[0]
				f = open(name+'.txt')
				content = f.read().split('+++')
				f.close()
				mark  = eval(content[FMARK].split(':')[1])	
				relations = content[FREL].split(':')[1][:-1]
				tt = content[FQTEXT] + '\n\\textbf{Answer:}\n' + content[FATEXT]
				if relations == '':
					tex += '\\question[%d] %s \\droppoints\n'%(mark, tt)
				else:
					tex += '\\question[%d](%s) %s \\droppoints\n'%(mark,relations,tt)
			tex += '\\end{questions}\n'

		tex += '\\hrule \\end{document}'
		return tex

	def processTex(self, tex, path):	#path will be filename or part directory name
		f1, f2 = os.path.split(path)
		if f2 == '':
			f2 = path[-3:-1]
		else:
			f2 = os.path.splitext(f2)[0]
			
		print(f1,f2)
		os.chdir(f1)
		
		texfile = f2 + '.tex'
		f = open(texfile, 'w')
		f.write(tex)
		f.close()	
	
		p1 = subprocess.Popen(["xelatex", texfile, '-interaction=nonstopmode'])
		p1.wait(timeout=3)
		os.unlink(f2 + '.log')
		os.unlink(f2 + '.aux')
		#os.unlink(f2 + '.tex')	
		
		pdf = f2 + '.pdf'
		
		dstpdf = os.path.join(self.courseDir, pdf)
		srcpdf = os.path.join(f1, pdf)
		if os.path.exists(dstpdf):
			os.unlink(dstpdf)
		os.replace(srcpdf, dstpdf)		# Move PDF to Course Directory
		return dstpdf
		

	def makeAllPDF(self):
		for k in range(1,9):
			folder = os.path.join(self.courseDir,'P%d/'%k)
			print (folder) 
			tex = self.generatePartTex(folder)		
			self.processTex(tex,folder)


	def makePartPDF(self):
		try:
			part = eval(GUI.partNumberEdit.text())
		except:
			GUI.dialog_critical('Part number NOT set.')
			return		
		if part < 1 or part > 8:
			GUI.dialog_critical('Invalid part number. Should be from 1 to 8.')
			return	
		folder = os.path.join(self.courseDir,'P%d'%part) + os.sep
		tex = self.generatePartTex(folder)
		dstpdf = self.processTex(tex, folder)
		p2 = subprocess.Popen(["evince", dstpdf])	

		
	def singlePDF(self):
		if self.file_save() != True:
			return

		tex = '\\documentclass[10pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.1in, top=0.1in, bottom=0.1in, right=0.1in]{geometry}\n'	

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'		
		
		tex += '\\begin{document} \\noindent'

		tex += '\\hrule \\vspace{0.05in}\n \\begin{questions}\n'
		text = open(self.path).read()
		tex += '\\question\n'+ text.split('+++')[5] 
		tex += '\n\\textbf{Answer:}\n\n'
		tex += text.split('+++')[6]
		tex += '\\end{questions}\\hrule \\end{document}'

		dstpdf = self.processTex(tex, self.path)
		p2 = subprocess.Popen(["evince", dstpdf])	

# File open, close, new file
			
	def file_open(self):
		if self.courseDir == None:
			GUI.dialog_critical('Set the Course Directory first.')
			return	
		dialog = QFileDialog()
		if self.path != None:
			dialog.setDirectory(self.path)
		else:
			dialog.setDirectory(self.courseDir)
		path, _ = dialog.getOpenFileName(GUI, "Open file", "",
							"Text files(*.txt);;Latex files(*.tex);;All Files(*.*)")
		if path == '':
			return
		cdir = os.path.dirname(os.path.dirname(path))
		if  cdir !=self.courseDir:
			GUI.dialog_critical('Opens files inside the Part directories inside the current Course directory.')
			return	
		
		print(path)
			
		try:
			f = open(path, 'r')
			content = f.read().split('+++')
			f.close()
		except Exception as e:
			GUI.dialog_critical(str(e))
		
		fn = os.path.split(path)[1]	
		qns = str(fn)[4:6]
		if qns[0] == '0':
			qns = qns.replace('0', '')
			
		GUI.qnLabel.setText('Qn = %s'%qns)
		self.questionNumber = eval(qns)
		print(self.questionNumber)
		status = content[FSTAT].split(':')[1].strip()
		if status == 'inactive':
			GUI.statusCB.setChecked(True)
		else:
			GUI.statusCB.setChecked(False)
			
		author = content[FAUTH].split(':')[1]
		GUI.authorEdit.setText(author)
		partnum = content[FPN].split(':')[1]
		GUI.partNumberEdit.setText(partnum)
		qtype = content[FTYPE].split(':')[1][:-1]
		GUI.questionTypeEdit.setText(qtype)
		relations = content[FREL].split(':')[1][:-1]
		GUI.relationsEdit.setText(relations)
		mark  = content[FMARK].split(':')[1]
		GUI.markEdit.setText(mark)
		qtext = content[FQTEXT]
		atext = content[FATEXT]				
		GUI.qeditor.setPlainText(qtext.strip())
		GUI.aeditor.setPlainText(atext.strip())
		self.path = path
		GUI.update_title("Editing %s"%self.path)		
		
	def getQuestionNumber(self):
		if self.courseDir == None:
			GUI.dialog_critical('Course Directory is NOT set.')
			return	
		wd = os.getcwd()
		try:
			part = eval(GUI.partNumberEdit.text())
		except:
			GUI.dialog_critical('Part number NOT set.')
			return	
		
		if part < 1 or part > 8:
			GUI.dialog_critical('Part number must be from 1 to 8.')
			return			
		qtype = GUI.questionTypeEdit.text().strip().upper()
		if qtype not in ['A', 'B', 'C']:
			GUI.dialog_critical('Question type must be A, B or C.')
			return	
			
		os.chdir(self.courseDir+'/P%d'%part)
		for qn in range(1,98):
			fn = 'P%d-%s%02d.txt'%(part, qtype, qn)
			print (fn)
			if not os.path.exists(fn):
				break
		os.chdir(wd)
		GUI.statusBar().showMessage('Next unused %s type question number is %d : %s'%(qtype,qn,fn))	
		return qn,fn	

	def file_save(self):
		if self.courseDir == None:
			GUI.dialog_critical('Course Directory is NOT set.')
			return 

		author = GUI.authorEdit.text().strip()
		if len(author) < 3:
			GUI.dialog_critical('Author details too short.')
			return 

		try:
			part = eval(GUI.partNumberEdit.text())
		except:
			GUI.dialog_critical('Part number NOT set.')
			return	
		if part < 1 or part > 8:
			GUI.dialog_critical('Invalid part number. Should be from 1 to 8.')
			return  		
					
		qtype = GUI.questionTypeEdit.text().strip().upper()
		if qtype not in ['A', 'B', 'C']:
			GUI.dialog_critical('Question type must be A, B or C.')
			return	

		relations = GUI.relationsEdit.text().strip()			
		if relations != '':
			try:
				rels = relations.split(',')
				nr = len(rels)
				relations = '' 		# make it afresh
				for k in range(nr):
					r = rels[k].strip()
					if r == '':
						continue
					rt = r[0].upper()
					if rt not in ['A', 'B', 'C']:
						GUI.dialog_critical('Relation must be A B or C.')
						return			
					rv = r[1:]
					if rv[0] == '0':		# removes one leading zero
						rv = rv[1:]
					rval = eval(rv)
					relations += '%s%d,'%(rt,rval)
				relations = relations[:-1]
			except:
				GUI.dialog_critical('Improper Relationship entry')
				return			
	
		if self.qpModel == 'M32':
			if qtype == 'A':
				mark = 1
			elif qtype == 'B':
				mark = 3
			elif qtype == 'C':
				mlimits = [2,6]
		elif self.qpModel == 'M40':
			if qtype == 'A':
				mark = 2
			elif qtype == 'B':
				mark = 4
			elif qtype == 'C':
				mlimits = [2,6]
		elif self.qpModel == 'M60':
			if qtype == 'A':
				mark = 3
			elif qtype == 'B':
				mark = 5
			elif qtype == 'C':
				mlimits = [3,10]
		elif self.qpModel == 'M80A':
			if qtype == 'A':
				mark = 4
			elif qtype == 'B':
				mark = 7
			elif qtype == 'C':
				mlimits = [3,13]
		elif self.qpModel == 'M80B':
			if qtype == 'A':
				mark = 4
			elif qtype == 'B':
				mark = 8
			elif qtype == 'C':
				mlimits = [3,12]
		elif self.qpModel == 'M48':
			if qtype == 'A':
				mark = 1
			elif qtype == 'B':
				mark = 5
			elif qtype == 'C':
				mlimits = [3,9]		

		if qtype == 'C':
			try:
				m = GUI.markEdit.text()
				print (m)
				mark = eval(m)
			except:
				GUI.dialog_critical('Mark not properly set.....')
				return
			if mark < mlimits[0] or mark > mlimits[1]:
				GUI.dialog_critical('Invalid mark %d. Min = %d. Max = %d'%(mark, mlimits[0],mlimits[1]))
				return

		fn = 'P%d-%s%02d.txt'%(part, qtype,self.questionNumber)
		fn = os.path.join(self.courseDir, 'P%d'%part, fn)
		self.path = fn

		#Do checking if new file .....
		wd = os.getcwd()						# move to the Part folder
		os.chdir(self.courseDir+'/P%d'%part)
		'''
		if os.path.exists(fn):
			res = QMessageBox.question(GUI, 'Saving File', "%s exists. Do you want to overwrite it?"%fn, 
				QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
			if res == QMessageBox.No:
				os.chdir(wd)
				return
		'''
		
		if GUI.statusCB.isChecked():
			status ='inactive'
		else:
			status = 'active'
		text = 'status:%s\n+++\nAuthor:%s\n+++\nPart Number:%d\n+++\nType:%s\n+++\nRelations:%s\n+++\nMark:%d\n+++\n'%(
		status,author, part, qtype,relations,mark)
		
		text += GUI.qeditor.toPlainText().strip()
		text += '\n+++\n'+GUI.aeditor.toPlainText().strip()
		try:
			f = open(fn, 'w')
			f.write(text)
		except Exception as e:
			GUI.dialog_critical(str(e))
		GUI.statusBar().showMessage('File %s saved to disk'%fn)		
		return True

	def newQuestion(self):
		if self.questionLocked == True:
			GUI.partNumberEdit.setReadOnly(False)
			GUI.questionTypeEdit.setReadOnly(False)
			self.questionLocked = False
			GUI.statusBar().showMessage('Enabled Part Number and Type Inputs. Click again after setting desired values')	
			return
			
		res = self.getQuestionNumber()
		if res == None:
			return
		qn, fn = res
		print (qn,fn)
		
		res = QMessageBox.question(GUI, 'New Question', 'Start new Question %s ?'%fn,
			QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
		if res != QMessageBox.Yes:
			return
		self.filename = fn
		self.questionNumber = qn
		s = 'Q# = %d'%qn
		GUI.qnLabel.setText(s)
		s = str(fn)
		GUI.statusBar().showMessage('Now Editing %s. Disabled Part# and QType input Fields'%s)
		GUI.relationsEdit.setText('')
		GUI.markEdit.setText('')

		GUI.partNumberEdit.setReadOnly(True)
		GUI.questionTypeEdit.setReadOnly(True)
		self.questionLocked = True
		
#----------------------- End of Class Course ------------------

class MainWindow(QMainWindow):   # Creating main window class
	#courseList = []
	def __init__(self, *args, **kwargs):
		super(MainWindow, self).__init__(*args, **kwargs)
		
		self.setGeometry(100, 100, 800, 600)			
		fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
		fixedfont.setPointSize(14)
		
		self.aeditor = QPlainTextEdit()
		self.aeditor.setFont(fixedfont)
		self.aeditor.setLineWrapMode(1)
		
		self.qeditor = QPlainTextEdit()
		self.qeditor.setFont(fixedfont)
		self.qeditor.setLineWrapMode(1)

		layout = QVBoxLayout()
		hbox = QHBoxLayout()
		hbox0 = QHBoxLayout()
		hbox1 = QHBoxLayout()
		hbox2 = QHBoxLayout()

		x = QLabel('Server Name ')
		hbox.addWidget(x)
		self.hostLineEdit = QLineEdit('openqp.scischool.in')
		self.hostLineEdit.setMaximumWidth(200)
		hbox.addWidget(self.hostLineEdit)
		x = QLabel('Database Name ')
		hbox.addWidget(x)
		self.dbLineEdit = QLineEdit('kuqbank')
		self.dbLineEdit.setMaximumWidth(200)
		hbox.addWidget(self.dbLineEdit)
		self.listButton = QPushButton('Connect to Database')
		self.listButton.clicked.connect(self.updateCourseList)
		hbox.addWidget(self.listButton)
		hbox.addStretch()

		self.projectDirButton = QPushButton('Project Directory: Not set')
		self.projectDirButton.clicked.connect(self.setProjectDir)
		hbox0.addWidget(self.projectDirButton)
		self.courseDirButton = QPushButton('Course Directory: Not set')
		self.courseDirButton.clicked.connect(self.setCourseDir)
		hbox0.addWidget(self.courseDirButton)
		#hbox0.addStretch()	

		self.openFileButton = QPushButton('Make Part PDF')
		self.openFileButton.clicked.connect(course.makePartPDF)
		hbox0.addWidget(self.openFileButton)
		self.openFileButton = QPushButton('Make All PDFs')
		self.openFileButton.clicked.connect(course.makeAllPDF)
		hbox0.addWidget(self.openFileButton)
			
		self.newQButton = QPushButton('New Question')
		self.newQButton.clicked.connect(course.newQuestion)
		hbox1.addWidget(self.newQButton)
		
		self.openFileButton = QPushButton('Open a Question')
		self.openFileButton.clicked.connect(course.file_open)
		hbox1.addWidget(self.openFileButton)

		self.saveButton = QPushButton('Save Question')
		self.saveButton.clicked.connect(course.file_save)
		hbox1.addWidget(self.saveButton)

		self.makePDFButton = QPushButton('Show PDF')
		self.makePDFButton.clicked.connect(course.singlePDF)
		hbox1.addWidget(self.makePDFButton)

		self.partNumberLabel = QLabel('Part#')
		self.partNumberEdit = QLineEdit(' ')
		self.partNumberEdit.setMaximumWidth(40)
		hbox2.addWidget(self.partNumberLabel)
		hbox2.addWidget(self.partNumberEdit)

		self.questionTypeLabel = QLabel('Type(A,B,C)')
		self.questionTypeEdit = QLineEdit('  ');
		self.questionTypeEdit.setMaximumWidth(40)
		hbox2.addWidget(self.questionTypeLabel)
		hbox2.addWidget(self.questionTypeEdit)

		self.qnLabel = QLabel('Q# not Set')
		hbox2.addWidget(self.qnLabel)
		hbox2.addStretch()

		self.markLabel = QLabel('Mark')
		self.markEdit = QLineEdit('  ')
		self.markEdit.setMaximumWidth(40)
		hbox2.addWidget(self.markLabel)
		hbox2.addWidget(self.markEdit)
		
		self.relationsLabel = QLabel('Relations')
		self.relationsEdit = QLineEdit('  ');
		self.relationsEdit.setMaximumWidth(200)
		hbox2.addWidget(self.relationsLabel)
		hbox2.addWidget(self.relationsEdit)

		self.authorLabel = QLabel('Author:')
		self.authorEdit = QLineEdit('   ')
		self.authorEdit.setMaximumWidth(200)
		hbox2.addWidget(self.authorLabel)
		hbox2.addWidget(self.authorEdit)

		self.statusCB = QCheckBox()
		self.statusCB.setText('Inactive')
		hbox2.addWidget(self.statusCB)
		#self.statusCB.setChecked(True)
		
		layout.addLayout(hbox)
		layout.addLayout(hbox0)
		layout.addLayout(hbox1)
		layout.addLayout(hbox2)
		
		split = QSplitter(Qt.Vertical)
		split.addWidget(self.qeditor)
		split.addWidget(self.aeditor)
		layout.addWidget(split)
		container = QWidget()
		container.setLayout(layout)
		self.setCentralWidget(container)  	# making it central widget
		
		self.status = QStatusBar()
		self.setStatusBar(self.status)
		
		edit_menu = self.menuBar().addMenu("&Edit")
		edit_menu.addAction('Undo',self.qeditor.undo)
		edit_menu.addAction('Redo',self.qeditor.redo)
		edit_menu.addAction('Cut',self.qeditor.cut)
		edit_menu.addAction('Copy',self.qeditor.copy)
		edit_menu.addAction('Paste',self.qeditor.paste)
		edit_menu.addAction('Select All',self.qeditor.selectAll)
		
		self.db_menu = self.menuBar().addMenu("&Select Remote Course")
		
		action_menu = self.menuBar().addMenu("&Actions")
		action_menu.addAction('Import Course from Database', course.importCourse)
		action_menu.addAction('Check Local Files', course.checkFiles)
		action_menu.addAction('Export Course to Database', course.exportCourse)
		self.show()
		
	def updateCourseList(self):
		self.courseList = []
		self.host = self.hostLineEdit.text()
		self.db = self.dbLineEdit.text()
		print('Connecting to %s on %s ...'%(self.db,self.host))		
	
		try:
			conn = psycopg2.connect(database=self.db, host=self.host, user='dummy', password="dum123")
			cursor = conn.cursor()
			sql =  'select * from coursedetails;'
			cursor.execute(sql)
			res = cursor.fetchall()
		except:
			self.dialog_critical('Connection to Database failed. Only Local Editing possible.')
			return		
		for k in res:
				self.courseList.append(k)
		print ('List=',self.courseList)
		self.db_menu.clear()
		for k in self.courseList:
			self.db_menu.addAction(k[0], lambda item=k[0]: course.selectRemoteCourse(item))				
		self.statusBar().showMessage('Connected to Database "%s".'%self.db)		
	
	def dialog_critical(self, s):
		dlg = QMessageBox(self)
		dlg.setText(s)
		dlg.setIcon(QMessageBox.Critical)
		dlg.show()
	
	def update_title(self, text):
		self.setWindowTitle(text)

	def getPassword(self, cname):
		text, ok = QInputDialog.getText(None, "Database %s"%cname, "Password?", 
		QLineEdit.Password)
		if ok and text:
			return text

	def setProjectDir(self):
		dialog = QFileDialog()
		s = dialog.getExistingDirectory(self, 'Select Project Directory')
		if s == '':
			return
		os.chdir(s)
		course.projectDir = s
		self.statusBar().showMessage('Project Directory set to %s'%s)		
		self.projectDirButton.setText('Project Directory : %s'%s)		

	def setCourseDir(self):
		if course.projectDir == None:
			self.dialog_critical('Set the Project Directory first.')
			return	
		dialog = QFileDialog()
		dialog.setDirectory(course.projectDir)		
		cname = dialog.getExistingDirectory(self, 'Select Course Directory')
		if cname == '':
			return
		if os.path.dirname(cname) !=course.projectDir:
			self.dialog_critical('Course directory must be inside the Project Directory.')
			return	
		if os.path.isdir(cname) == False:
			reply = QMessageBox.question(self, 'Create Directory', 'Do you want to create a new Course Directory?',
			QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
			if reply == QMessageBox.No:
				return	course
			os.mkdir(cname) 
			for k in range(1,9):
				os.mkdir('P%d'%k)
				self.statusBar().showMessage('Created new Course Directory %s and P1 to P8 inside that.'%cname)
		course.courseDir = cname
		os.chdir(course.courseDir)
		try:
			self.setProperties()
		except:
			self.dialog_critical('Failed to set course properties. Check the properties file.')
						
		course.courseCode = os.path.split(cname)[1]	
		self.courseDirButton.setText('Course Directory : %s'%course.courseCode)
		self.statusBar().showMessage('Course Directory set to %s'%cname)
		
		#Clear the editor
		self.path = None
		GUI.update_title('')		
		GUI.authorEdit.setText('')
		GUI.partNumberEdit.setText('')
		GUI.questionTypeEdit.setText('')
		GUI.relationsEdit.setText('')
		GUI.markEdit.setText('')
		GUI.qeditor.setPlainText('')
		GUI.aeditor.setPlainText('')

	def setProperties(self):
		ss = open('properties').read().split('\n')
		for row in ss:
			sss = row.split(':')
			if sss[0] == 'language':
				course.language = sss[1]
			elif sss[0] == 'qpModel':
				course.qpModel = sss[1]
			elif sss[0] == 'year':
				course.year = sss[1]

app = QApplication(sys.argv)   # Main program
course = Course()
GUI = MainWindow()
app.exec_()		# main loop




