#!/usr/bin/python3

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtPrintSupport import *

from shutil import rmtree
import subprocess, os, sys, time
from pathlib import Path
import psycopg2
import numpy as np
from random import randint, shuffle
from datetime import datetime, timedelta
from shutil import copyfile

myhost = '13.232.225.206'
myhost = 'localhost' 

mydb = 'cuqbank'
QTEXT = 0	# indices for the rows from the database
QATEXT = 1
QAUTHOR = 2
QMODN = 3
QTYPE = 4
QSN = 5
QRELATIONS = 6
QMARK = 7
QSTAT = 8
QFIGNAME = 9
QFIG = 10
QUSED = 11    	# added by this program
QPART2 = 12   	# second question of C
SINGLEQL = 12	# length of the list after adding the 'used' flag

#offsets of the fields in the question text file
FSTAT = 0
FAUTH = 1
FPN = 2
FTYPE = 3
FREL = 4
FMARK = 5
FQTEXT = 6
FATEXT = 7

NFRAGS = 8		# text file contains N sections separated by +++
MAXMOD = 8		# Maximum 8 modules
qStatusTypes = ['medium','easy','tough','inactive']

# For Devagiri College
qpdataM70DC = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2,'DUR':120}

# For CU
qpdataM70CU = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2,'DUR':120}
qpdataM50CU = {'Ma':2,'Mb':6,'Mc':10,'Na':8,'Nb':4,'Nc':1,'Ta':10,'Tb':5,'Tc':2,'DUR':90}

# Kannur Univ colleges
qpdataM70KU = {'Ma':3,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':2,'Ta':8,'Tb':6,'Tc':3,'DUR':120}
qpdataM50KU = {'Ma':2,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':1,'Ta':8,'Tb':6,'Tc':2,'DUR':90}

# Models below are for KU departments only
qpdataM32KU = {'Ma':1,'Mb':3,'Mc':6, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM40KU = {'Ma':2,'Mb':3,'Mc':7, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM48KU = {'Ma':1,'Mb':5,'Mc':9, 'Na':6,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM60KU = {'Ma':3,'Mb':5,'Mc':10,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM80KU = {'Ma':4,'Mb':7,'Mc':13,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}

class Qpgen():		# Generates Question papers, from local data
	# Arrays to store the number of questions in each Part
	naq = None		
	nbq = None
	ncq = None
	# 2D Arrays to store the questions from each module
	Atypes = None
	Btypes = None
	Ctypes = None

	def __init__(self):
		self.qpHeader = 'Header'
		self.qpCode = 'qpcode'
		self.examMonth = 'December'
		self.examYear = '2025'
		self.qpErr = ''
		
	def copyTexFigs(self,tex):
		fragments = tex.split('\\includegraphics')
		nfig = len(fragments)
		res = []
		for k in range(1,nfig):
			s1 = fragments[k]
			s2 = s1.split('{')[1]
			fig = s2.split('}')[0]
			print('Copied to outputs : ', fig)
			copyfile(fig, os.path.join(course.outputDir,fig))
			
	def loadQuestions(self): # load questions from the module directories
		print('Entering loadQuestions')
		oldwd = os.getcwd()	
		courseFolder = course.projectDir + '/' + course.courseCode   
		os.chdir(courseFolder)	# change to the desired Course code dir
		
		results = []	# List of lists to store all the questions
		for module in range(1, course.numModules + 1):	# from all modules
			wd = str(Path.cwd()) + '/' + 'M%d'%module
			os.chdir(wd)				# change the desired Module dir
			files = Path('.').glob('*.txt')
			for fn in files:
				f = open(fn)
				content = f.read().split('+++')
				f.close()
				if len(content) != NFRAGS:
					print('Question file Error: ',content)
				row = []
				row.append(content[FQTEXT].strip())
				row.append(content[FATEXT].strip())
				row.append(content[FAUTH].split(':')[1].strip())
				row.append(eval(content[FPN].split(':')[1]))
				row.append(content[FTYPE].split(':')[1].strip())
				qsn = str(fn)[4:6]
				if qsn[0] == '0':
					qsn = qsn.replace('0', '')
				row.append(eval(qsn))
				row.append(content[FREL].split(':')[1].strip())
				row.append(eval(content[FMARK].split(':')[1]))
				row.append(content[FSTAT].split(':')[1].strip())
				row.append('None')  # figname
				row.append('None')  # figure
				tt = content[FQTEXT] + content[FATEXT]
				self.copyTexFigs(tt)
				results.append(row)
			os.chdir('..')		
		os.chdir(oldwd)
		
		# Count the number of questions in each section, type wise
		self.naq = np.zeros(MAXMOD, dtype=int)
		self.nbq = np.zeros(MAXMOD, dtype=int)
		self.ncq = np.zeros(MAXMOD, dtype=int)

		for row in results:
			for k in range(MAXMOD):		# k goes from 0 to MAXMOD-1. Add 1 while comparing
				if row[QTYPE] == 'A' and row[QMODN] == k+1:
					self.naq[k] += 1
				if row[QTYPE] == 'B' and row[QMODN] == k+1:
					self.nbq[k] += 1
				if row[QTYPE] == 'C' and row[QMODN] == k+1:
					self.ncq[k] += 1
		
		#print('NQs ',self.naq, self.nbq, self.ncq)
		# Make 3 array of objects, size MAXMOD, to transfer the SQL query results
		self.Atypes = np.empty(MAXMOD,object)
		self.Btypes = np.empty(MAXMOD,object)
		self.Ctypes = np.empty(MAXMOD,object)

		for k in range(MAXMOD):	# Array size = MAXMOD x NoQ in each (variable)
			self.Atypes[k] = np.empty(self.naq[k], object)
			self.Btypes[k] = np.empty(self.nbq[k], object)
			self.Ctypes[k] = np.empty(self.ncq[k], object)

		for k in range(len(results)):	# Copy to 3 2D Array of objects
			row = list(results[k])
			row.append(False)	# Add the 'QUSED' flag, set False			
			if row[QSTAT] == 'inactive':
				row[QUSED] = True
			if row[QSTAT] == 'easy' and qpLevel == 'tough':
				row[QUSED] = True
			if row[QSTAT] == 'tough' and qpLevel == 'easy':
				row[QUSED] = True
			if row[QTYPE] == 'A':
				sec = row[QMODN]
				qn = row[QSN]
				self.Atypes[sec-1][qn-1] = row	# Indices from 0 to MAXMOD-1 (not 1 to MAXMOD)
			if row[QTYPE] == 'B':
				sec = row[QMODN]
				qn = row[QSN]
				self.Btypes[sec-1][qn-1] = row
			if row[QTYPE] == 'C':
				sec = row[QMODN]
				qn = row[QSN]
				self.Ctypes[sec-1][qn-1] = row

		print('Loaded %d Questions '%len(results))		
		print(self.naq, self.nbq, self.ncq)
		print('Exiting loadQuestions')
		
	#--------------- Select Questions for QP ---------------------
	def markRelations(self,module,relation): # Mark the specified questions as used
		if relation == '':
			return
		rels = relation.split(',')
		for r in rels:
			r = r.strip()
			rtype = r[0]
			rval = r[1:]
			rqn = eval(r[1:])-1
			if rtype == 'C':
				self.Ctypes[module][rqn][QUSED] = True
			if rtype == 'B':
				self.Btypes[module][rqn][QUSED] = True
			if rtype == 'A':
				self.Atypes[module][rqn][QUSED] = True

	def showDist(self, ia,ib,ic):
		Ma,Mb,Mc = self.qpd['Ma'],self.qpd['Mb'],self.qpd['Mc']
		Ta,Tb,Tc = self.qpd['Ta'],self.qpd['Tb'],self.qpd['Tc']
		t = Ta*Ma +Tb*Mb +Tc*Mc
		#print(self.moduleMarks, self.pm2, )
		pat = '<h3>Report:</h3>\n'
		pat += '%s has %d Modules. Marks ['%(course.courseCode[6:],course.numModules)
		tm = 0
		for k in range(course.numModules):
			pat += '%3.1f '%course.moduleMarks[k]
			tm += course.moduleMarks[k]
		pat += '] = %d.<br>\n'%tm
		pat += 'All Questions %dx%d + %dx%d + %dx%d = %d marks. '%(Ta,Ma,Tb,Mb,Tc,Mc,t)
		pat += 'Scaling gives ['
		for k in range(course.numModules):
			pat += '%3.1f '%self.pm2[k]
		pat += ']<br><br>\n'
		pat += 'Module wise distribution of the generated paper(with the deviation):<br>\n'
		 
		mtotal = 0
		mark = ''
		for k in range(course.numModules):
			mp = 0
			pat += 'Questions from Module %d | '%(k+1)
			if k in ic:
				nc = ic.count(k)
				mp += nc * self.qpd['Mc']
				pat += '%s'%('C'*nc)
			if k in ib:
				nb = ib.count(k)
				mp += nb * self.qpd['Mb']
				pat += '%s'%('B'*nb)
			if k in ia:
				na = ia.count(k)
				mp += na * self.qpd['Ma']
				pat += '%s'%('A'*na)
			pat += ' : %d(%3.1f)<br>\n'%(mp,(mp-self.pm2[k]))
			mtotal += mp
			mark += '%d:'%mp
		#pat += ': Total %2d marks<br>\n'%mtotal
		#pat += mark
		f = open(os.path.join(course.outputDir,'pat.txt'),'w')
		f.write(pat)	
		print(pat)

	#------------------------different selection models----------------------
	def selectM70DC(self):
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = course.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		for k in range(nmods):
			course.moduleMarks[k] = 70/nmods	# Assign equal marks to all Modules
		tot = 0
		self.pm2 = []
		for k in range(nmods):
			x = course.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x	
				
		ic = []
		n = randint(0, nmods/2 -1)*2   # 0, 2, 4 or 6
		ic.append(n)		# select Two C types  
		ic.append(n+1)	

		ib = []
		ia = []
		for k in range(nmods):		# Add 6 B and A each
			if k not in ic: 		
				ib.append(k)
				ia.append(k)
				
		while 1:		
			n = randint(0, nmods-1)	
			if n not in ic and ib.count(n) < 2 : 
				ib.append(n)
			if len(ib) == Tb:
				break

		while 1:		
			n = randint(0, nmods-1)	
			if n not in ic and ib.count(n) + ia.count(n) < 3 : 
				ia.append(n)
			if len(ia) == Ta:
				break

		print (ic,ib,ia)
		return ia,ib,ic
	
	def selectM70KU(self):
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = course.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		for k in range(course.numModules):
			course.moduleMarks[k] = 70/course.numModules	# Equal marks to all Modules
		tot = 0
		self.pm2 = []
		for k in range(course.numModules):
			x = course.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x		
			
		ic = []
		n = randint(0, nmods/2 -1)*2   # 0, 2, 4 or 6
		ic.append(n)		# Two C types from the same 
		ic.append(n)	
		while 1:
			n = randint(0, nmods/2 -1)*2   # 0, 2, 4 or 6			
			if n not in ic:
				ic.append(n)			# Add the third C type
				break

		Csel = [ic[0], ic[0]+1, ic[2], ic[2]+ 1]   # odd and even modules selected for C

		# b only from half, excluding C fully, a from noC half. also two from C3 top
		# print (ic,Csel)

		ib = []
		for k in range(nmods):		# Add Bs, excluding the C blocks
			if k not in Csel: 
				ib.append(k)
		while 1:		
			n = randint(0, nmods-1)	
			if (n not in Csel) and ib.count(n) < 2 : 
				ib.append(n)
			if len(ib) == Tb:
				break
				
		ia = [ic[2], ic[2]]   		 	# Two from the third C type module
		for k in range(nmods):			# One each from the no C blocks
			if k not in Csel:
				ia.append(k)
		while 1:		
			n = randint(0, nmods-1)	
			if n not in Csel and ia.count(n) + ib.count(n) < 3: 
				ia.append(n)
			if len(ia) >= Ta:
				break
								
		print (ic,ib,ia)		
		return ia,ib,ic

	def selectM50KU(self):  # For 50 marks, modules to pick A,B and C types.
		NMODS50 = 6			# Only the first Six modules
		ic = []
		ic.append(randint(0, NMODS50-1))	 # pick from 6 modules only
		if ic[0] % 2:  		# odd number
			ic.append(ic[0] - 1)
		else:
			ic.append(ic[0] + 1)

		ib = []
		for k in range(NMODS50):
			if k not in ic:
				ib.append(k)
		for k in range(0,NMODS50,2):		
				if k not in ic:
					ib.append(k + randint(0,1))
		
		ia = []
		for k in range(NMODS50):
			if k not in ic:
				ia.append(k)
				ia.append(k)
		return ia,ib,ic
		
	def selectM60KU(self):   # Ma=3, Mb=5, Mc=10 , Ta=6, Tb=5, Tc=5
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = course.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		for k in range(course.numModules):
			course.moduleMarks[k] = 60/course.numModules	# Assign equal marks to all Modules
		tot = 0
		self.pm2 = []
		for k in range(course.numModules):
			x = course.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x		
			
		ic = []				# Algorithm given by TKM
		for k in range(0,MAXMOD,2):
			ic.append(randint(0,1)+k)
		while 1:
			n = randint(0,MAXMOD-1)
			if n not in ic:
				ic.append(n)
				break
		ib = []
		for p in range(MAXMOD):
			if p not in ic:
				ib.append(p)
		shuffle(ib)
		
		ia = ic.copy() 			# Five A types  from the same sections
		ia.append(ib[-1])		# Add third section of B to A	
		ib = (ib*2)[:-1]		# repeat the first two sections of B type 
		return ia,ib,ic	
			
	def selectM70CU(self):   # for CU
		'''
		Ma=3, Mb=6, Mc=10 , Ta=10, Tb=8, Tc=2 and
		Ma=2, Mb=6, Mc=10 , Ta=10, Tb=5, Tc=2 
		'''		
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = course.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		tot = 0
		self.pm2 = []
		for k in range(course.numModules):
			x = course.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x

		trial = 0
		ic = []			# fill C types, handle sections without C also
		while len(ic) < Tc and trial < 10:			
			n = randint(0,nmods-1)  
			trial += 1
			if n not in ic:
				ic.append(n)
				#print(trial)

		ib = []
		for n in range(nmods):	# fill B 1 round(exclude C filled)
			k = n % nmods
			mark = Mb * ib.count(k) + Mc * ic.count(k)
			if Mb+mark <= self.pm2[k] and len(ib) < Tb:   
				ib.append(k)
			
		ia = []	
		# fill remaining as per the moduleMarks values
		for n in range(50):		
			k = n%nmods #randint(0,nmods-1)
			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Mb+mark <= self.pm2[k]*1.1 and len(ib) < Tb and k not in ic):
				ib.append(k)
			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Ma+mark <= self.pm2[k]*1.1 and len(ia) < Ta):
				ia.append(k)
			if len(ia) == qpd['Ta'] and len(ib) == qpd['Tb'] and len(ic) == qpd['Tc']:
				break
			
		if len(ia) != qpd['Ta'] or len(ib) != qpd['Tb'] or len(ic) != qpd['Tc']:
			self.qpErr = 'Failed to fill all %d sections '%nmods
		return ia,ib,ic
			
	def selectM50CU(self):   # Common for several qpModels
		'''
		Ma=3, Mb=6, Mc=10 , Ta=10, Tb=8, Tc=2 and
		Ma=2, Mb=6, Mc=10 , Ta=10, Tb=5, Tc=2 
		'''		
		qpd = self.qpd
		Ma,Mb,Mc = qpd['Ma'],qpd['Mb'],qpd['Mc']
		Na,Nb,Nc = qpd['Na'],qpd['Nb'],qpd['Nc']
		Ta,Tb,Tc = qpd['Ta'],qpd['Tb'],qpd['Tc']
		nmods = course.numModules
		ratio = (Ta*Ma +Tb*Mb +Tc*Mc)/(Na*Ma +Nb*Mb +Nc*Mc)
		tot = 0
		self.pm2 = []
		for k in range(course.numModules):
			x = course.moduleMarks[k]*ratio
			self.pm2.append(x)
			tot += x

		trial = 0
		ic = []			# fill C types, handle sections without C also
		while len(ic) < Tc and trial < 10:			
			n = randint(0,nmods-1)  
			trial += 1
			if n not in ic:
				ic.append(n)
				#print(trial)

		ib = []
		for n in range(nmods):	# fill B 1 round(exclude C filled)
			k = n % nmods
			mark = Mb * ib.count(k) + Mc * ic.count(k)
			if Mb+mark <= self.pm2[k] and len(ib) < Tb:   
				ib.append(k)
			
		ia = []	
		# fill remaining as per the moduleMarks values
		for n in range(50):		
			k = n%nmods #randint(0,nmods-1)
			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Mb+mark <= self.pm2[k]*1.1 and len(ib) < Tb and k not in ic):
				ib.append(k)
			mark = Ma * ia.count(k) + Mb * ib.count(k) + Mc * ic.count(k)
			if(Ma+mark <= self.pm2[k]*1.1 and len(ia) < Ta):
				ia.append(k)
			if len(ia) == qpd['Ta'] and len(ib) == qpd['Tb'] and len(ic) == qpd['Tc']:
				break
			
		if len(ia) != qpd['Ta'] or len(ib) != qpd['Tb'] or len(ic) != qpd['Tc']:
			self.qpErr = 'Failed to fill all %d sections '%nmods
		return ia,ib,ic
						
	def selectQuestions(self):
		self.qpErr = ''
		if course.qpModel == 'M70DC':
			self.qpd = qpdataM70DC
			ia,ib,ic = self.selectM70DC()				
		elif course.qpModel == 'M70CU':
			self.qpd = qpdataM70CU
			ia,ib,ic = self.selectM70CU()				
		elif course.qpModel == 'M50CU':
			self.qpd = qpdataM50CU
			ia,ib,ic = self.selectM50CU()				
		elif course.qpModel == 'M70KU':
			self.qpd = qpdataM70KU
			ia,ib,ic = self.selectM70KU()	
		elif course.qpModel == 'M50KU':
			self.qpd = qpdataM50KU
			ia,ib,ic = self.selectM50KU()	
		elif course.qpModel == 'M60KU':  # KU department only
			self.qpd = qpdataM60KU
			ia,ib,ic = self.selectM60KU()	
		else:	
			self.qpErr = 'Unsupported Question paper type.'
			return
		ia.sort()		
		ib.sort()
		#Lists to store the selected questions
		self.AQ = []
		self.BQ = []
		self.CQ = []	
		print ('IABC = ', ia, ib, ic)
		self.showDist(ia, ib, ic)
		
		self.MAXMARK = self.qpd['Ma'] * self.qpd['Na'] + self.qpd['Mb']\
		 * self.qpd['Nb'] +	self.qpd['Mc'] * self.qpd['Nc']
		 
		for k in range(len(ic)):		# Select Ctype first
			module = ic[k]				# Pre-selected module #s are stored in ic
			shuffle(self.Ctypes[module])
			for q in self.Ctypes[module]:
				if q[QUSED] == False:
					q[QUSED] = True
					self.CQ.append(q)
					print('First C: ', module, q[QSN], q[QMARK])
					self.markRelations(module, q[QRELATIONS])
					if	q[QMARK] != self.qpd['Mc']:		# get one more piece 
						foundQ2 = False
						mr2 = self.qpd['Mc'] - q[QMARK]
						#print ('CM mr mr2 = ', q[QMARK],mr2)
						shuffle(self.Ctypes[module])
						for q2 in self.Ctypes[module]:
							if q2[QMARK] == mr2 and q2[QUSED] == False:
								q2[QUSED] = True
								foundQ2 = True
								self.CQ[-1].append(q2)	# next piece
								print('Second C: ', module, q2[QSN], q2[QMARK])
								self.markRelations(module, q[QRELATIONS])
								break
						if foundQ2 == False:
							self.qpErr = 'No Second part:%d %d %d'%(module, q[QSN], q[QMARK])
							return
					break
					
		for k in range(len(ib)):	# Select Ctype first
			module = ib[k]			
			shuffle(self.Btypes[module])		
			for q in self.Btypes[module]:
				if q[QUSED] == False:
					q[QUSED] = True
					self.BQ.append(q)
					print('B', q[QMARK])
					#self.markRelations(module, q[QRELATIONS])
					break
		
		for k in range(len(ia)):	# Select A type
			module = ia[k]			
			shuffle(self.Atypes[module])
			for q in self.Atypes[module]:
				if(q == None):
					print ('Q is None',self.Atypes[module])
				if q[QUSED] == False:
					q[QUSED] = True
					self.AQ.append(q)
					print('A', q[QMARK])
					#self.markRelations(module, q[QRELATIONS])
					break

		#Verify the results.
		print(self.AQ)
		if len(self.AQ) != len(ia):
			self.qpErr = 'A type Mismatch: len(AQ) = %d. len(ia) = %d'%(len(self.AQ), len(ia))
			return
		if len(self.BQ) != len(ib):
			self.qpErr = 'B type Mismatch: len(BQ) = %d. len(ib) = %d'%(len(self.BQ), len(ib))
			return
		if len(self.CQ) != len(ic):
			self.qpErr = 'C type Mismatch: len(CQ) = %d. len(ic) = %d'%(len(self.CQ), len(ic))
			return	
		
		print('End: selectQuestions', len(self.AQ), len(self.BQ), len(self.CQ))
		# End of selectQuestions
		
	def questionTex(self):
		now = datetime.now() + timedelta(hours = 5, minutes=30)
		self.dt = now.strftime("%d/%m/%Y %H:%M:%S")

		#print('Python: Answer Done. Making QP..')
		#--------Generate Latex document, using data from AQ,BQ and CQ -----
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if course.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif course.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
		elif course.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic,Scale=1.1]{Scheherazade}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 
		elif course.language.lower() == 'urdu':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{urdu}\n'
			tex += '\\setmainfont[Script=urdu,Scale=1.1]{nafees}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 

		tex += '\\begin{document}\n'
		
		tex += '\\begin{center}\n'
		tex += 'Reg No:.................. \\hfill \\small {%s}\n\n'%(self.qpCode)
		tex += 'Name :.................... \\hfill  \\small {%s}\n\n'%('')	
		
		tex += '\\large\n \\textbf {%s Semester %s %s Examination\n'%(course.semester, course.degree, course.subject)
		tex += '\\\\%s %s\n'%(self.examMonth, self.examYear)
		courseYear = course.courseCode[1:5]
		tex += '(%s Admission onwards)\\\\\n'%courseYear # syllabusYear
		tex += '%s (%s)}\n\n'%(course.courseCode[8:],course.courseName)
		tex += '(%s)\n\n'%self.qpHeader
		tex += '\\vspace{0.1in}\n\\end{center}\n '
		tex += '\\hrule\n\\vspace{0.1in}\n'
	
		tex += '\\noindent\n' 
		tex += 'Time : %3d min  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(self.qpd['DUR'], self.MAXMARK)
		
		tex += '{\\noindent \\bfseries  Part A (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Na'], self.qpd['Ma'])

		tex += '\\begin{questions} \\pointsdroppedatright\n'

		if course.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.AQ:
			tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
			tex += '\\vspace{0.05in}\n'
		if course.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '{\\noindent \\small \\bfseries  Part B (Answer any %d questions. Each carries %d marks) }\n'%(self.qpd['Nb'], self.qpd['Mb'])

		if course.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.BQ:
			tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
			tex += '\\vspace{0.05in}\n'
		if course.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
		
		tex += '{\\noindent\n\\small \\bfseries Part C (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nc'], self.qpd['Mc'])
		
		if course.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'		
		for q in self.CQ:
			if len(q) == SINGLEQL:
				tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
				tex += '\\vspace{0.05in}\n'
			else:
				q2 = q[QPART2]
				tex += '\\question\n'
				tex += '\\begin{parts} \n'
				tex += '\\part[%d] %s \\droppoints\n'%(q[QMARK], q[QTEXT].strip())
				tex += '\\part[%d] %s \\droppoints\n'%(q2[QMARK], q2[QTEXT].strip())
				tex += '\\end{parts}\n'	
				tex += '\\vspace{0.05in}\n'
		if course.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
		tex += '\\end{questions}\n'
		tex += '\\end{document}'
		return tex

	def answerTex(self):
		#print('Python: Entering answerTex..')
		#--Generate Latex document, using questions & answers from AQ,BQ and CQ -----
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if course.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif course.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'		
		elif course.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic, Scale=1.1]{Scheherazade}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 

		tex += '\\begin{document} \\noindent\n'

		tex += '%s \\hfill %s\n\\newline '%(self.qpCode, self.dt)
		
		tex += '\\begin{center} \\large\n \\textbf {%s Semester %s %s Examination\n'%(course.semester, course.degree, course.subject)
		tex += '%s %s\n'%(self.examMonth, self.examYear)
		tex += '(%s Admission onwards)\\\\\n'%course.syllabusYear
		tex += '%s (%s)}\n\\end{center}\n'%(course.courseCode[8:], course.courseName)
		
		tex += '\\noindent\n' 
		tex += 'Time : %s  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(self.qpd['DUR'], self.MAXMARK)
		tex += '\\hrule\n\\vspace{0.1in}\n'

		tex += '{\\noindent \\bfseries  Part A (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Na'], self.qpd['Ma'])
		tex += '\\begin{questions} \\pointsdroppedatright\n'

		if course.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.AQ:
			tex += '\\question[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODN],q[QSN],q[QTEXT].strip())
			
			if course.language.lower() == 'arabic':
				tex += '\\textbf{إجابة}\n\n'
			else:
				tex += '\\textbf{Answer:}\n\n'
			
			tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
			tex += '\\vspace{0.05in}\n'
		if course.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '{\\noindent \\small \\bfseries  Part B (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nb'], self.qpd['Mb'])

		if course.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.BQ:
			tex += '\\question[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODN],q[QSN],q[QTEXT].strip())
			
			if course.language.lower() == 'arabic':
				tex += '\\textbf{إجابة}\n\n'
			else:
				tex += '\\textbf{Answer:}\n\n'
			
			tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
			tex += '\\vspace{0.05in}\n'
		if course.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
			
		tex += '{\\noindent\n\\small \\bfseries Part C (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nc'], self.qpd['Mc'])

		if course.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.CQ:
			if len(q) == SINGLEQL:
				tex += '\\question[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODN],q[QSN],q[QTEXT].strip())
				
				if course.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'
				
				tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
				tex += '\\vspace{0.05in}\n'
			else:
				q2 = q[QPART2]
				tex += '\\question\n'
				tex += '\\begin{parts} \n'
				tex +='\\part[%d]M%d(%d):%s\n\n'%(q[QMARK],q[QMODN],q[QSN],q[QTEXT].strip())
				if course.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'

				tex += '%s\\droppoints\n'%q[QATEXT].strip()				
				tex += '\\part[%d]M%d(%d): %s\n\n'%(q2[QMARK],q2[QMODN],q2[QSN],q2[QTEXT].strip())
				
				if course.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'

				tex += '%s\\droppoints\n'%q2[QATEXT].strip()
				tex += '\\end{parts}'	
				tex += '\\vspace{0.05in}\n'
		if course.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '\\end{questions}\n'
		tex += '\\end{document}'
		return tex

	def showQP(self, c):
		if course.courseCode == None:
			GUI.dialog_critical('No Course is selected.')
			return 
		
		self.loadQuestions()	
		self.selectQuestions()

		if self.qpErr != '':
			GUI.dialog_critical(self.qpErr)
			return 
			
		qtex = self.questionTex()
		atex = self.answerTex()
		qfn = course.courseCode
		afn = course.courseCode +'_ANS'

		qpdf = course.processTex(qtex, qfn)
		apdf = course.processTex(atex, afn)

		p1 = subprocess.Popen(["evince", os.path.join(course.outputDir,qpdf)])	
		p2 = subprocess.Popen(["evince", os.path.join(course.outputDir,apdf)])	
	# End of Class Qpgen


class Course:
	def __init__(self):
		self.password = None
		self.remoteCourseCode = None	# set by selectRemoteCourse()
		self.subject = None
		self.language = 'english'
		self.courseCode = None
		self.projectDir = None
		self.outputDir = None
		self.courseDir = None
		self.filepath = None				# default file name
		self.author = ''
		self.moduleNumber = None
		self.numModules = None
		self.moduleMarks = [None]*MAXMOD
		self.qType = None
		self.relations = ''
		self.mark = None
		self.editingQuestion = False
		self.questionNumber = None

	#-------------- LaTex and PDF file generation ----------------- 
	def generateModuleTex(self, modnum):	# Generates LaTex document for one Module	
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
		elif self.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic]{Scheherazade}\n'
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 					
		tex += '\\begin{document} \\noindent'

		oldwd = os.getcwd()	
		os.chdir(os.path.join(course.courseDir, 'M%d'%modnum))
		mask = 'M%d-A*.txt'%modnum
		files = sorted(Path('.').glob(mask))
		print(mask, files)
		tex += '\\par\n{Module %s Questions }\n'%mask[1:2]
		tex += '\\hrule \\vspace{0.05in}\n \\par {Section A}\n'
		
		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'		
		if len(files) != 0:
			tex += '\\begin{questions}\n\\pointsdroppedatright\n'
				
			for fn in files:	
				print(fn)	
				name = os.path.splitext(fn)[0]
				f = open(name+'.txt')
				content = f.read().split('+++')
				f.close()
				mark  = eval(content[FMARK].split(':')[1])	
				relations = content[FREL].split(':')[1][:-1] 
				if self.language.lower() == 'arabic':			
					qa = content[FQTEXT] + '\n\\textbf{إجابة}\n\n' + content[FATEXT]	
				else:
					qa = content[FQTEXT] + '\n\\textbf{Answer:}\n\n' + content[FATEXT]	
				qp.copyTexFigs(qa)
				
				if relations == '':
					rel = ''
				else:
					rel = '(Relation: %s)'%relations
				tex += '\\question[%d]%s\n %s'%(mark,rel,qa)			
				tex += '\\droppoints\n'

			tex += '\\end{questions}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '\\hrule \\vspace{0.05in}\n \\par {Section B}\n'

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'

		mask = 'M%d-B*.txt'%modnum
		files = sorted(Path('.').glob(mask))

		if len(files) != 0:
			tex += '\\begin{questions}\n'
			for fn in files:		
				name = os.path.splitext(fn)[0]
				f = open(name+'.txt')
				content = f.read().split('+++')
				f.close()
				mark  = eval(content[FMARK].split(':')[1])	
				relations = content[FREL].split(':')[1][:-1]
				
				if self.language.lower() == 'arabic':			
					qa = content[FQTEXT] + '\n\\textbf{إجابة}\n\n' + content[FATEXT]	
				else:
					qa = content[FQTEXT] + '\n\\textbf{Answer:}\n\n' + content[FATEXT]	
				qp.copyTexFigs(qa)
				
				if relations == '':
					rel = ''
				else:
					rel = '(Relation: %s)'%relations
				tex += '\\question[%d]%s\n %s'%(mark,rel,qa)			
				tex += '\\droppoints\n'
			tex += '\\end{questions}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
		
		tex += '\\hrule \\vspace{0.05in}\n \\par {Section C}\n'
		
		mask = 'M%d-C*.txt'%modnum
		files = sorted(Path('.').glob(mask))

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		
		if len(files) != 0:
			tex += '\\begin{questions}\n'
			for fn in files:		
				name = os.path.splitext(fn)[0]
				f = open(name+'.txt')
				content = f.read().split('+++')
				f.close()
				mark  = eval(content[FMARK].split(':')[1])	
				relations = content[FREL].split(':')[1][:-1]			

				if self.language.lower() == 'arabic':			
					qa = content[FQTEXT] + '\n\\textbf{إجابة}\n\n' + content[FATEXT]	
				else:
					qa = content[FQTEXT] + '\n\\textbf{Answer:}\n\n' + content[FATEXT]	
				qp.copyTexFigs(qa)
				
				if relations == '':
					rel = ''
				else:
					rel = '(Relation: %s)'%relations
				tex += '\\question[%d]%s\n %s'%(mark,rel,qa)			
				tex += '\\droppoints\n'

			tex += '\\end{questions}\n'
			
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
			
		tex += '\\hrule \\end{document}'
		os.chdir(oldwd)
		return tex

	def processTex(self, tex, fn):	# accepts Tex matter and filename.returns pdf
		oldwd = os.getcwd()	
		os.chdir(course.outputDir)
		#for x in Path(".").glob("*.pdf") : x.unlink()		

		open(fn + '.tex', 'w').write(tex)
		p1 = subprocess.Popen(["xelatex", fn+'.tex', '-interaction=nonstopmode'])
		p1.wait(timeout=3)

		for x in Path(".").glob("*.aux") : x.unlink()		
		for x in Path(".").glob("*.log") : x.unlink()		
		#for x in Path(".").glob("*.tex") : x.unlink()		
		#for x in Path(".").glob("*.png") : x.unlink()		
		
		os.chdir(oldwd)		
		return fn + '.pdf'
		
	def makeAllPDF(self):
		if self.courseCode == None:
			GUI.dialog_critical('No Course selected yet.')
			return
		for k in range(1,self.numModules + 1):
			tex = self.generateModuleTex(k)		
			self.processTex(tex, 'M%d'%k)
		GUI.dialog_critical('PDF outputs written to %s'%course.outputDir)

	def showModulePDF(self):
		if self.courseCode == None:
			GUI.dialog_critical('No Course selected yet.')
			return
		if self.moduleNumber == None:
			GUI.dialog_critical('Module number NOT set.')
			return		
		tex = self.generateModuleTex(self.moduleNumber)
		pdf = self.processTex(tex, 'M%d'%self.moduleNumber)
		p2 = subprocess.Popen(["evince", os.path.join(course.outputDir,pdf)])	
		
	def singlePDF(self):
		if self.file_save() != True:
			return
		tex = '\\documentclass[10pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.1in, top=0.1in, bottom=0.1in, right=0.1in]{geometry}\n'	

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
		elif self.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic]{Scheherazade}\n'
		elif course.language.lower() == 'urdu':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{urdu}\n'
			tex += '\\setmainfont[Script=Arabic, Language=Urdu, Scale=1.1] {NafeesNastaleeq.ttf}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 
											
		tex += '\\begin{document} \\noindent'

		tex += '\\hrule \\vspace{0.05in}\n \\begin{questions}\n'
		text = open(self.filepath).read()
		tex += '\\question\n'+ text.split('+++')[FMARK] 
		tex += text.split('+++')[FQTEXT]
		tex += '\nAnswer\n'
		tex += text.split('+++')[FATEXT]
		tex += '\\end{questions}\\hrule \\end{document}'
		qp.copyTexFigs(tex)
		print(tex)
		
		fn = 'M%d-%s%02d'%(self.moduleNumber, self.qType, self.questionNumber)
		pdf = self.processTex(tex, fn)
		p2 = subprocess.Popen(["evince", os.path.join(course.outputDir,pdf)])	

	
	#------------ Remote course selection, Import and Export ----------
	
	def selectRemoteCourse(self, code):  # select a Course from DB
		pw = GUI.getPassword(code)	
		if pw == None:
			GUI.dialog_critical('No password obtained for %s.'%code)	
			return
		try:
			conn = psycopg2.connect(database=GUI.db, host=GUI.host,
				user=code.lower(), password=pw)				
			cursor = conn.cursor()	
			cursor.execute("SELECT count(*) FROM %s"%code)	
			res = cursor.fetchone()
		except:
			GUI.dialog_critical('Connection to database %s failed.'%code)
			return
		GUI.msg.setText('Selected %s.Found %d records'%(code,res[0]))
		self.coursepassword = pw		# Use later for import / export
		self.remoteCourseCode = code
		self.language = 'english'
		for c in GUI.courseList:
			if c[0] == code:
				self.courseName = c[3]
				self.syllabusYear = c[1]
				self.inst = c[2]
				self.subject = c[4]
				self.degree = c[5]
				self.semester = c[6]
				self.language = c[7]
				self.qpModel = c[8]
				self.duration = c[9]
				self.moduleMarkString = c[10]
				self.numModules = eval(c[10][0])
		GUI.RCselectButton.setText('Remote: '+code)
		GUI.moduleNumberMenu.clear()
		for qt in range(1,MAXMOD+1):
			s = '%d'%qt
			GUI.moduleNumberMenu.addAction(s, lambda item=s: GUI.setModuleNumber(item))
		GUI.moduleNumberButton.setMenu(GUI.moduleNumberMenu)
		#print(self.moduleMarkString)

		
	def importCourse(self):	
		if self.projectDir == None:
			GUI.dialog_critical('Set the Project Directory first.')
			return	

		if self.remoteCourseCode == None:
			GUI.dialog_critical('No Course selected from Database yet.')
			return	
			
		# Bring all questions from the database
		db = GUI.db
		host = GUI.host
		#print (db, host, self.coursepassword)		
		try:
			conn = psycopg2.connect(database=db, host=host, user=self.remoteCourseCode.lower(), password=self.coursepassword)
			cursor = conn.cursor()	
			cursor.execute("SELECT * FROM %s"%self.remoteCourseCode)	
			dbresults = cursor.fetchall()		
		except:
			GUI.dialog_critical('Import from Database %s failed.'%self.remoteCourseCode)
			return		
		
		cdir = os.path.join(self.projectDir, self.remoteCourseCode)	
		if os.path.isdir(cdir) == False:
			msg = 'Local Project Directory %s does not have %s. Do you want to create it?'%(self.projectDir, self.remoteCourseCode)
			reply = QMessageBox.question(GUI, 'Create Course Directory', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
			if reply == QMessageBox.No:
				return
		else:
			msg = 'Directory %s exists, contents will be deleted. Do you want to proceed?'%cdir
			reply = QMessageBox.question(GUI, 'Create Course Directory', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
			if reply == QMessageBox.No:
				return
			rmtree(cdir)		# Delete the local one
		
		os.chdir(self.projectDir)				
		os.mkdir(self.remoteCourseCode) 
		os.chdir(self.remoteCourseCode) 
		#print('import :',self.numModules)
		for k in range(1,self.numModules + 1):
			os.mkdir('M%d'%k)
		self.courseDir = cdir
		os.chdir(self.courseDir)
		self.courseCode = self.remoteCourseCode				
		GUI.courseDirButton.setText('Course code : %s'%self.courseCode)				
		self.filepath = None	
		
		# Make the properties file
		pf = open('properties', 'w')
		pf.write('institude:%s\n'%self.inst)
		pf.write('syllabusyear:%s\n'%self.syllabusYear)
		pf.write('courseName:%s\n'%self.courseName)
		pf.write('subject:%s\n'%self.subject)
		pf.write('degree:%s\n'%self.degree)
		pf.write('semester:%s\n'%self.semester)
		pf.write('language:%s\n'%self.language)
		pf.write('qpModel:%s\n'%self.qpModel)
		pf.write('duration:%s\n'%self.duration)
		pf.write('numModules:%s\n'%self.moduleMarkString)
		pf.close()
		
		nr = len(dbresults)
		for k in range(nr):				# Save the results to the Course directory
			row = dbresults[k]
			module = row[QMODN]      
			os.chdir('M%d'%module)		# Change to folder Pn
			if row[QFIGNAME] != None:
				open(row[QFIGNAME], 'wb').write(row[QFIG])	
			else:
				if row[QSTAT] == 'inactive':
					matter = 'status:inactive\n+++\n'
				else:
					matter = 'status:active\n+++\n'	
				matter = 'status:%s\n+++\n'%row[QSTAT]
					
				matter += 'Author:%s\n+++\nModule Number:%d\n+++\nType:%s\n+++\n'%(row[QAUTHOR],row[QMODN],row[QTYPE])
				matter += 'Relations:%s\n+++\nmark:%d\n+++\n%s\n+++\n%s\n'%(row[QRELATIONS],row[QMARK], row[QTEXT], row[QATEXT])
				fn = 'M%d-%s%02d.txt'%(row[QMODN], row[QTYPE], row[QSN])
				f = open(fn,'w')
				f.write(matter)
				f.close()
			os.chdir('..')	
		count = 0
		for row in dbresults:
			if row[QTEXT] != None:
				count += 1	
				
		GUI.statusBar().showMessage('Downloaded %d Records'%count)	
		GUI.showStatus()
		GUI.update_title('')		
		self.author = None
		GUI.authorEdit.setText('')
		self.moduleNumber = None
		GUI.moduleNumberButton.setText('Module #')
		self.qType = None
		GUI.questionTypeButton.setText('Q Type')
		self.relations = ''
		GUI.relationsEdit.setText('')
		self.mark = None
		GUI.markEdit.setText('')
		GUI.qeditor.setPlainText('')
		GUI.aeditor.setPlainText('')
		self.editingQuestion = False
		self.questionNumber = None		
		GUI.showStatus()
		
		# end importCourse()

	def verifyFiles(self):				# verify the format of files
		if self.courseDir == None:
			GUI.dialog_critical('Select a Course Diectory first.')
			return	
		count = 0
		os.chdir(self.courseDir)
		print('verify: numModules= ',self.numModules)
		for module in range(1,self.numModules+1):		# traverse all Module directories
			wd = str(Path.cwd()) + '/' + 'M%d'%module
			os.chdir(wd)	# change directory to the desired Module
			files = Path('.').glob('*.txt')
			ana = []		# store question numbers for error checking
			bna = []
			cna = []
			for fn in files:
				fn = str(fn)
				f = open(fn)
				filetext = f.read()
				content = filetext.split('+++')	
				f.close()
				if len(fn) != 10:
					GUI.dialog_critical('Module %d: %s .Filename format is wrong.'%(module, fn))
					os.chdir('..')	
					return
					
				p = '%d'%module
				if fn[1] != p:
					GUI.dialog_critical('%s should not be inside M%d'%(fn,module))
					os.chdir('..')	
					return
									
				if len(content) != NFRAGS:
					GUI.dialog_critical('module %d: File %s should have %d "+++" separated fields'%(module,fn,self.numModules))
					return
					
				try:
					modulenum = eval(content[FPN].split(':')[1])
					if modulenum != module:
						GUI.dialog_critical('Error in %s : module %d inside M%d'%(fn, modulenum,module))
				except:
					GUI.dialog_critical('%s :Module number parse error'%fn)
					return
				
				try:
					qns = fn[4:6]
					if qns[0] == '0':
						qns = qns.replace('0', '')
					qnum = eval(qns)
				except:
					GUI.dialog_critical('%s: Question number parse error'%fn)
					return

				try:
					qtype = content[FTYPE].split(':')[1][:-1]
					if qtype == 'A':
						ana.append(qnum)
					elif qtype == 'B':
						bna.append(qnum)
					elif qtype == 'C':
						cna.append(qnum)
					else:
						GUI.dialog_critical('%s : Invalid question type'%fn)
						return								
				except:
					GUI.dialog_critical('%s : Question type parse error'%fn)
					return		

				try:
					mark  = eval(content[FMARK].split(':')[1])
					if mark < 2 or mark > 14:                  # check for all types  
						GUI.dialog_critical('%s : Invalid mark'%fn)
						return
				except:
					GUI.dialog_critical('%s : Mark parse error'%fn)
					return

				author = content[FAUTH].split(':')[1]
				if len(author) < 3:
					GUI.dialog_critical('%s : Author info too short'%fn)
					return
				
				relations = content[FREL].split(':')[1].strip()
				if relations != '':
					try:
						rels = relations.split(',')
						nr = len(rels)
						relations = '' 		# make it afresh
						for k in range(nr):
							r = rels[k].strip()
							if r == '':
								continue
							rt = r[0].upper()
							if rt not in ['A', 'B', 'C']:
								GUI.dialog_critical('Relation must be A B or C.')
								return			
							rv = r[1:]
							if rv[0] == '0':		# removes one leading zero
								rv = rv[1:]
							rval = eval(rv)
							rfn = 'M%d-%s%02d.txt'%(module,rt,rval)
							if os.path.exists(rfn) == False:
								GUI.dialog_critical('Related Question of "%s" "%s" not found'%(fn, rfn))
								return
							relations += '%s%d,'%(rt,rval)
						relations = relations[:-1]
					except:
						GUI.dialog_critical('Improper Relationship entry')
						return								
				
				qtext = content[FQTEXT].strip()
				if len(qtext) < 3:
					GUI.dialog_critical('%s : Question too short'%fn)
					return
				
				#if "'" in qtext:print('Question contains single quotes', fn)

				atext = content[FATEXT].strip()

				#if "'" in atext: print('Answer contains single quotes', fn)

				fragments = qtext.split('\\includegraphics')
				nfig = len(fragments)
				for k in range(1,nfig):
					s1 = fragments[k]
					s2 = s1.split('{')[1]
					fig = s2.split('}')[0]
					if os.path.isfile(fig) == False:
						GUI.dialog_critical('%s : include file %s NOT found'%(fn,fig))
				count += 1		# End of loop

			for k in range(1, len(ana) + 1):
				fn = 'M%d-A%02d.txt'%(module,k)
				if os.path.exists(fn) == False:
					GUI.dialog_critical('Module %d: Question "%s" not found'%(module,fn))
					return
					
			for k in range(1, len(bna) + 1):
				fn = 'M%d-B%02d.txt'%(module,k)
				if os.path.exists(fn) == False:
					GUI.dialog_critical('Module %d: Question "%s" not found'%(module,fn))
					return

			for k in range(1, len(cna) + 1):
				fn = 'M%d-C%02d.txt'%(module,k)
				if os.path.exists(fn) == False:
					GUI.dialog_critical('Module %d: Question "%s" not found'%(module,fn))
					return		
			os.chdir('..')		# End of loop
		GUI.msg.setText('Verified %d Files'%count)
		return True
	
	def exportCourse(self):
		if self.verifyFiles() != True:
			GUI.dialog_critical('Local Files verification failed.')
			return
		if self.remoteCourseCode == None:
			GUI.dialog_critical('Remote Course not selected.')
			return	
		if self.courseCode == None:
			GUI.dialog_critical('Course Directory not set.')
			return	
		if self.courseCode != self.remoteCourseCode:
			GUI.dialog_critical('Course Codes NOT matching.')
			return	
			
		wd = os.getcwd()	
		target = self.courseCode

		db = GUI.db
		host = GUI.host
		pw = self.coursepassword
		#print (db, host, pw)		
		
		code = self.courseCode.lower()
		conn = psycopg2.connect(database=db, host=host, user=code, password=pw)
		cursor = conn.cursor()
		cursor.execute("DELETE from %s"%code)
		conn.commit()

		courseFolder = self.projectDir + '/' + self.courseCode   # Course Directory
		os.chdir(courseFolder)		# change directory to the desired Course code
		count = 0
		for module in range(1,self.numModules + 1):		# traverse M1 to M8
			wd = str(Path.cwd()) + '/' + 'M%d'%module
			os.chdir(wd)	# change directory to the desired Module
			files = Path('.').glob('*.txt')
			for fn in files:
				f = open(fn)
				content = f.read().split('+++')
				f.close()
				print(str(fn))
				qns = str(fn)[4:6]
				if qns[0] == '0':
					qns = qns.replace('0', '')
				qnum = eval(qns)
				status = content[FSTAT].split(':')[1].strip()
				author = content[FAUTH].split(':')[1].strip()
				moduleNum = eval(content[FPN].split(':')[1])
				qtype = content[FTYPE].split(':')[1].strip()
				relations = content[FREL].split(':')[1].strip()
				try:
					mark  = eval(content[FMARK].split(':')[1])
				except:
					GUI.dialog_critical('Invalid mark %s'%(content[FMARK]))
					return
				qtext = content[FQTEXT].strip()
				qtext = qtext.replace("'","''")		# Takes care of single quotes in SQL query
				atext = content[FATEXT].strip()
				#print ('status = ', status)
				sql = "insert into %s (qtext,atext,author,modnum,qtype,qnum,relations,mark,status) values ('%s', '%s', '%s', %d, '%s', %d, '%s', %d, '%s');"%(target,qtext,atext,author,moduleNum,qtype,qnum,relations,mark,status)
				#print (sql)
				try:
					cursor.execute(sql)	
					count += 1						# Question record count
				except:
					GUI.dialog_critical('SQL insert failed after %d Records'%count)
					os.chdir(wd)
					return
				# load image if any
				tt = qtext + atext
				fragments = tt.split('\\includegraphics')
				nfig = len(fragments)
				for k in range(1,nfig):
					s1 = fragments[k]
					s2 = s1.split('{')[1]
					fig = s2.split('}')[0]
					print(s1, s2, fig)
					if 1:
						drawing = open(fig, 'rb').read()
						img = psycopg2.Binary(drawing)
						sql = "insert into %s(modnum,figname, figure) values (%d,'%s', %s);"%(target,moduleNum,fig, img)
						cursor.execute(sql)
					else:
						GUI.dialog_critical('%s : Insert "%s" failed.'%(fn,fig))
						print(sql)
						os.chdir(wd)
						return
			os.chdir('..')		
		os.chdir(wd)
		conn.commit()
		GUI.dialog_critical('Uploaded %d Records'%count)
		#End of exportCourse() 

# -------------- File open, close, new Question -----------------------
	def file_save(self):
		if self.courseDir == None:
			GUI.dialog_critical('Course Directory is NOT set.')
			return 
		if self.editingQuestion == False:
			GUI.dialog_critical('No question to save. Create/Open first.".')
			return

		#status = GUI.authorEdit.text().strip()
		
		author = GUI.authorEdit.text().strip()
		if len(author) < 3:
			GUI.dialog_critical('Author details too short.')
			return 

		relations = GUI.relationsEdit.text().strip()			
		if relations != '':
			try:
				rels = relations.split(',')
				nr = len(rels)
				relations = '' 		# make it afresh
				for k in range(nr):
					r = rels[k].strip()
					if r == '':
						continue
					rt = r[0].upper()
					if rt not in ['A', 'B', 'C']:
						GUI.dialog_critical('Relation must be A B or C.')
						return			
					rv = r[1:]
					if rv[0] == '0':		# removes one leading zero
						rv = rv[1:]
					rval = eval(rv)
					relations += '%s%d,'%(rt,rval)
				relations = relations[:-1]
			except:
				GUI.dialog_critical('Improper Relationship entry')
				return	
		
		try:
			m = GUI.markEdit.text()
			mark = int(eval(m))
		except:
			GUI.dialog_critical('Mark not properly set.....')
			return						
		if self.qType == 'A' and mark != 3:
			GUI.dialog_critical('A type mark should be 3.')
			return

		if self.qType == 'B' and mark != 6:
			GUI.dialog_critical('B type mark should be 6.')
			return

		if self.qType == 'C' and mark not in [7,14]:
			GUI.dialog_critical('KU: C type mark should be 7 or 14.')
			return
						
		fn = 'M%d-%s%02d.txt'%(self.moduleNumber, self.qType,self.questionNumber)
		fn = os.path.join(self.courseDir, 'M%d'%self.moduleNumber, fn)
		self.filepath = fn

		wd = os.getcwd()						# move to the Module folder
		os.chdir(self.courseDir+'/M%d'%self.moduleNumber)
		
		text = 'status:%s\n+++\nAuthor:%s\n+++\nModule Number:%d\n+++\nType:%s\n+++\nRelations:%s\n+++\nMark:%d\n+++\n'%(
		course.qStatus,author, self.moduleNumber, self.qType,relations,mark)
		
		text += GUI.qeditor.toPlainText().strip()
		text += '\n+++\n'+GUI.aeditor.toPlainText().strip()
		try:
			f = open(fn, 'w')
			f.write(text)
		except Exception as e:
			GUI.dialog_critical(str(e))
		GUI.statusBar().showMessage('File %s saved to disk'%fn)		
		return True
			
	def file_open(self):
		if self.courseDir == None:
			GUI.dialog_critical('Set the Course Directory first.')
			return	
		dialog = QFileDialog()
		if self.filepath != None:
			dialog.setDirectory(self.filepath)
		else:
			dialog.setDirectory(self.courseDir)
		path, _ = dialog.getOpenFileName(GUI, "Open file", "",
							"Text files(*.txt);;Latex files(*.tex);;All Files(*.*)")
		if path == '':
			return
		cdir = os.path.dirname(os.path.dirname(path))
		if  cdir !=self.courseDir:
			GUI.dialog_critical('File is outside the Course directory.')
			return	
			
		try:
			f = open(path, 'r')
			content = f.read().split('+++')	# Read and split file text
			f.close()
		except Exception as e:
			GUI.dialog_critical(str(e))
			return
			
		pn = content[FPN].split(':')[1][0]   # Take only one character
		self.moduleNumber = eval(pn)
		GUI.moduleNumberButton.setText('Module '+pn)
		self.filepath = path
		
		fn = os.path.split(path)[1]	
		qns = str(fn)[4:6]
		if qns[0] == '0':
			qns = qns.replace('0', '')
		self.questionNumber = eval(qns)		
		GUI.qNumLabel.setText('Qn = %s'%qns)

		self.qType = content[FTYPE].split(':')[1][0]
		GUI.questionTypeButton.setText(self.qType+ ' Type')

		qstat = content[FSTAT].split(':')[1].strip()
		course.qStatus = qstat
		GUI.qStatusTypeButton.setText(qstat)
			
		author = content[FAUTH].split(':')[1]
		GUI.authorEdit.setText(author)
		relations = content[FREL].split(':')[1][:-1]
		GUI.relationsEdit.setText(relations)
		mark  = content[FMARK].split(':')[1]
		GUI.markEdit.setText(mark)
		GUI.qeditor.setPlainText(content[FQTEXT].strip())
		GUI.aeditor.setPlainText(content[FATEXT].strip())
		GUI.update_title("Editing %s"%self.filepath)	
		GUI.moduleNumberButton.setEnabled(False) 
		GUI.questionTypeButton.setEnabled(False) 
		self.editingQuestion = True		
		
	def getQuestionNumber(self):
		'''
		Scan through the existing question files in the specified Module directory and returns the next available number.
		'''
		wd = os.getcwd()
		if self.moduleNumber == None:
			GUI.dialog_critical('Module number NOT set.')
			return	
		if self.qType == None:
			GUI.dialog_critical('Question Type NOT set.')
			return	
			
		os.chdir(self.courseDir+'/M%d'%self.moduleNumber)
		for qn in range(1,98):
			fn = 'M%d-%s%02d.txt'%(self.moduleNumber, self.qType, qn)
			if not os.path.exists(fn):
				break
		os.chdir(wd)
		GUI.statusBar().showMessage('Next unused %s type question number is %d : %s'%(self.qType,qn,fn))	
		return qn,fn	

	def newQuestion(self):
		if self.courseDir == None:
			GUI.dialog_critical('Course Directory is NOT set.')
			return	
		if self.editingQuestion == True:
			GUI.dialog_critical('Close the currently open Question.')
			return
			
		res = self.getQuestionNumber()
		if res == None:
			return
		qn, fn = res
		
		res = QMessageBox.question(GUI, 'New Question', 'Start new Question %s ?'%fn,
			QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
		if res != QMessageBox.Yes:
			return
		self.filepath = fn
		self.questionNumber = qn
		s = 'Q# = %d'%qn
		GUI.qNumLabel.setText(s)
		s = str(fn)
		GUI.statusBar().showMessage('Now Editing %s. Disabled Module# and QType input Fields'%s)
		GUI.relationsEdit.setText('')
		#GUI.markEdit.setText('')

		GUI.moduleNumberButton.setEnabled(False) 
		GUI.questionTypeButton.setEnabled(False) 
		self.editingQuestion = True
		GUI.update_title("Editing %s"%self.filepath)	

	def closeQuestion(self):
		if self.editingQuestion == False:
			return
		res = QMessageBox.question(GUI, 'Save Question', 'Save question before closing ?',
			QMessageBox.Yes | QMessageBox.No, QMessageBox.Yes)
		if res == QMessageBox.Yes:
			if self.file_save() != True:
				return
		
		#self.author = None
		#GUI.authorEdit.setText('')
		#self.moduleNumber = None
		#GUI.moduleNumberButton.setText('Module #')
		#self.qType = None
		#GUI.questionTypeButton.setText('Q Type')
		#self.mark = None
		#GUI.markEdit.setText('')
		
		self.relations = ''
		GUI.relationsEdit.setText('')
		GUI.qeditor.setPlainText('')
		GUI.aeditor.setPlainText('')
		self.questionNumber = None		
		GUI.qNumLabel.setText('Qn# ??')
		self.editingQuestion = False
		GUI.moduleNumberButton.setEnabled(True) 
		GUI.questionTypeButton.setEnabled(True) 
		GUI.update_title('')	
		
#----------------------- End of Class Course ------------------

class MainWindow(QMainWindow):   # Creating main window class
	#courseList = []
	subFilter = 'All'
	semFilter = 'All'
	subjectFilter = ['All', 'Mathematics', 'EVS', 'Statistics', 'Physics', 'Philosophy', 'Hindi', 'Malayalam', 'Arabic']
	semesterFilter = ['All','First', 'Second', 'Third', 'Fourth', 'Fifth', 'Sixth', 'Seventh', 'Eighth']

	def __init__(self, *args, **kwargs):
		super(MainWindow, self).__init__(*args, **kwargs)
		self.setGeometry(100, 100, 1100, 800)			
		fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
		fixedfont.setPointSize(14)
		
		self.aeditor = QPlainTextEdit()
		self.aeditor.setFont(fixedfont)
		self.aeditor.setLineWrapMode(1)
		
		self.qeditor = QPlainTextEdit()
		self.qeditor.setFont(fixedfont)
		self.qeditor.setLineWrapMode(1)

		layout = QVBoxLayout()
		hbox = QHBoxLayout()
		hbox0 = QHBoxLayout()
		hbox1 = QHBoxLayout()
		hbox2 = QHBoxLayout()
		hbox3 = QHBoxLayout()
		hbox4 = QHBoxLayout()
		layout.addLayout(hbox)
		layout.addLayout(hbox0)
		layout.addLayout(hbox1)
		layout.addLayout(hbox2)

		x = QLabel('Server')
		hbox.addWidget(x)
		self.hostLineEdit = QLineEdit(myhost)
		self.hostLineEdit.setMaximumWidth(150)
		hbox.addWidget(self.hostLineEdit)
		
		self.dbConnectButton = QPushButton('Connect to DB->')
		hbox.addWidget(self.dbConnectButton)
		self.dbConnectButton.clicked.connect(self.updateCourseList)	
		self.dbLineEdit = QLineEdit(mydb)
		self.dbLineEdit.setMaximumWidth(150)
		hbox.addWidget(self.dbLineEdit)	

		self.filterSubjectButton = QPushButton('Subject: All')
		hbox.addWidget(self.filterSubjectButton)
		m = QMenu()
		for sub in self.subjectFilter:
			m.addAction(sub, lambda item=sub: self.setSubjectFilter(item))
		self.filterSubjectButton.setMenu(m)
		
		self.filterSemesterButton = QPushButton('Semester: All')
		hbox.addWidget(self.filterSemesterButton)
		m = QMenu()
		for sub in self.semesterFilter:
			m.addAction(sub, lambda item=sub: self.setSemesterFilter(item))
		self.filterSemesterButton.setMenu(m)
					
		self.RCmenu = QMenu()
		self.RCselectButton = QPushButton('Select Remote Course')
		#self.RCselectButton.clicked.connect(self.updateCourseList)
		self.RCselectButton.setMenu(self.RCmenu)
		hbox.addWidget(self.RCselectButton)
		
		actionMenu = QMenu()
		self.actionSelectButton = QPushButton('Actions')
		#self.actionSelectButton.clicked.connect(self.updateCourseList)
		self.actionSelectButton.setMenu(actionMenu)
		hbox.addWidget(self.actionSelectButton)
		actionMenu.addAction('Import Course from Database', course.importCourse)
		actionMenu.addAction('Export Course to Database', course.exportCourse)	

		self.qpButton = QPushButton('make QP')
		self.qpButton.clicked.connect(qp.showQP)
		hbox.addWidget(self.qpButton)
		hbox.addStretch()

		self.projectDirButton = QPushButton('Project Directory: Not set')
		self.projectDirButton.clicked.connect(self.setProjectDir)
		hbox0.addWidget(self.projectDirButton)
		self.courseDirButton = QPushButton('Course Directory: Not set')
		self.courseDirButton.clicked.connect(self.selectLocalCourse)
		hbox0.addWidget(self.courseDirButton)
		self.verifyButton = QPushButton('Verify Files')
		self.verifyButton.clicked.connect(course.verifyFiles)
		hbox0.addWidget(self.verifyButton)

		self.openFileButton = QPushButton('Show Module PDF')
		self.openFileButton.clicked.connect(course.showModulePDF)
		hbox0.addWidget(self.openFileButton)
		self.openFileButton = QPushButton('Make All PDFs')
		self.openFileButton.clicked.connect(course.makeAllPDF)
		hbox0.addWidget(self.openFileButton)
			
		self.newQButton = QPushButton('New Question')
		self.newQButton.clicked.connect(course.newQuestion)
		hbox1.addWidget(self.newQButton)
		
		self.openFileButton = QPushButton('Open a Question')
		self.openFileButton.clicked.connect(course.file_open)
		hbox1.addWidget(self.openFileButton)

		self.saveButton = QPushButton('Save Question')
		self.saveButton.clicked.connect(course.file_save)
		hbox1.addWidget(self.saveButton)

		self.closeButton = QPushButton('Close Question')
		self.closeButton.clicked.connect(course.closeQuestion)
		hbox1.addWidget(self.closeButton)

		self.makePDFButton = QPushButton('Show PDF')
		self.makePDFButton.clicked.connect(course.singlePDF)
		hbox1.addWidget(self.makePDFButton)

		self.moduleNumberButton = QPushButton('Module #')
		hbox2.addWidget(self.moduleNumberButton)
		self.moduleNumberMenu = QMenu()
		for qt in range(1,MAXMOD+1):
			s = '%d'%qt
			self.moduleNumberMenu.addAction(s, lambda item=s: self.setModuleNumber(item))
		self.moduleNumberButton.setMenu(self.moduleNumberMenu)

		self.qNumLabel = QLabel('Qn# ??')
		hbox2.addWidget(self.qNumLabel)

		self.questionTypeButton = QPushButton('Q Type')
		hbox2.addWidget(self.questionTypeButton)
		menu = QMenu()
		for qt in ['A', 'B', 'C']:
			menu.addAction(qt, lambda item=qt: self.setQtype(item))
		self.questionTypeButton.setMenu(menu)
		
		self.markLabel = QLabel('Mark')
		self.markEdit = QLineEdit('  ')
		self.markEdit.setMaximumWidth(30)
		hbox2.addWidget(self.markLabel)
		hbox2.addWidget(self.markEdit)

		self.relationsLabel = QLabel('Relations')
		self.relationsEdit = QLineEdit('  ');
		self.relationsEdit.setMaximumWidth(150)
		hbox2.addWidget(self.relationsLabel)
		hbox2.addWidget(self.relationsEdit)

		self.authorLabel = QLabel('Author:')
		self.authorEdit = QLineEdit('   ')
		self.authorEdit.setMaximumWidth(200)
		hbox2.addWidget(self.authorLabel)
		hbox2.addWidget(self.authorEdit)

		course.qStatus = 'medium'
		self.qStatusTypeButton = QPushButton('Level: medium')
		hbox2.addWidget(self.qStatusTypeButton)
		menu = QMenu()
		for qs in qStatusTypes:
			menu.addAction(qs, lambda item=qs: self.setqStatus(item))
		self.qStatusTypeButton.setMenu(menu)
		hbox2.addStretch()
				
		split = QSplitter(Qt.Vertical)
		split.addWidget(self.qeditor)
		split.addWidget(self.aeditor)
		layout.addWidget(split)
		
		# Display the Course Information
		self.courseInfoLabel = QLabel('')
		self.courseInfoLabel.setStyleSheet("border: 1px solid blue;") 
		hbox3.addWidget(self.courseInfoLabel)
		layout.addLayout(hbox3)

		self.msg = QLabel('')
		hbox4.addWidget(self.msg)
		layout.addLayout(hbox4)

		container = QWidget()
		container.setLayout(layout)
		self.setCentralWidget(container)  	# making it central widget
			
		edit_menu = self.menuBar().addMenu("&Edit")
		edit_menu.addAction('Undo',self.qeditor.undo)
		edit_menu.addAction('Redo',self.qeditor.redo)
		edit_menu.addAction('Cut',self.qeditor.cut)
		edit_menu.addAction('Copy',self.qeditor.copy)
		edit_menu.addAction('Paste',self.qeditor.paste)
		edit_menu.addAction('Select All',self.qeditor.selectAll)
		self.show()
		
	def setModuleNumber(self, pn):
		self.moduleNumberButton.setText('Module ' + pn)
		course.moduleNumber = int(pn)
		
	def setQtype(self, q):
		self.questionTypeButton.setText(q+' Type')
		course.qType = q

	def setqStatus(self, qstat):
		self.qStatusTypeButton.setText('Level: '+qstat)
		course.qStatus = qstat

	def setSubjectFilter(self,s):
		self.subFilter = s
		self.filterSubjectButton.setText('Subject:'+s)
		self.fillRCmenu()

	def setSemesterFilter(self,s):
		self.semFilter = s
		self.filterSemesterButton.setText('Semester:'+s)
		self.fillRCmenu()

	def fillRCmenu(self):			
		self.RCmenu.clear()
		for row in self.courseList:
			print (row)
			if self.subFilter != 'All' and self.subFilter != row[4]:  	
				continue		
			if self.semFilter != 'All' and self.semFilter != row[6]:  	
				continue
			self.RCmenu.addAction(row[0], lambda item=row[0]: course.selectRemoteCourse(item))	
						
	def updateCourseList(self):
		self.courseList = []
		self.host = self.hostLineEdit.text()
		self.db = self.dbLineEdit.text()	
		try:
			print('Connecting to %s on %s ...'%(self.db,self.host), end =' ')	
			sys.stdout.flush()	
			conn = psycopg2.connect(database=self.db, host=self.host, user='dummy', password="dum123")
			cursor = conn.cursor()
			sql =  'select * from coursedetails order by code;'
			cursor.execute(sql)
			res = cursor.fetchall()
			print ('Fetched data\n')
			sys.stdout.flush()	
		except:
			self.dialog_critical('Connection to Database failed. Only Local Editing possible.')
			return		
		for k in res:
				self.courseList.append(k)
		self.msg.setText('Connected to Database. Found %d Courses.'%len(self.courseList))
		self.fillRCmenu()
	

	def dialog_critical(self, s):
		dlg = QMessageBox(self)
		dlg.setText(s)
		dlg.setIcon(QMessageBox.Critical)
		dlg.show()
	
	def update_title(self, text):
		self.setWindowTitle(text)

	def getPassword(self, cname):
		text, ok = QInputDialog.getText(None, "Database %s"%cname, "Password?", 
		QLineEdit.Password)
		if ok and text:
			return text

	def setProjectDir(self):
		dialog = QFileDialog()
		s = dialog.getExistingDirectory(self, 'Select Project Directory')
		if s == '':
			return
		os.chdir(s)
		course.projectDir = s
		self.projectDirButton.setText('Project Directory : %s'%s)	
		outdir = s+'/output'
		if not os.path.isdir(outdir):
			os.mkdir(outdir)	
		course.outputDir = outdir	
		self.msg.setText('Project Directory set to %s'%s)		

	def selectLocalCourse(self):
		if course.projectDir == None:
			self.dialog_critical('Set the Project Directory first.')
			return	
		dialog = QFileDialog()
		dialog.setDirectory(course.projectDir)		
		cname = dialog.getExistingDirectory(self, 'Select the Course for Editing')
		if cname == '':
			return
		if os.path.dirname(cname) !=course.projectDir:
			self.dialog_critical('Course directory must be inside the Project Directory.')
			return	
		course.courseDir = cname
		course.filepath = None
		os.chdir(course.courseDir)
										
		course.courseCode = os.path.split(cname)[1]	
		self.courseDirButton.setText('Course : %s'%course.courseCode)
		self.msg.setText('Course Directory set to %s'%cname)
		
		#Initialize the editor fields
		self.filepath = None
		self.update_title('')		
		self.editingQuestion = False
		self.authorEdit.setText('')
		course.moduleNumber = None
		self.moduleNumberButton.setText('Module #')
		course.questionNumber = None
		self.qNumLabel.setText('Q# ??')
		course.qType = None
		self.questionTypeButton.setText('Q Type')
		self.markEdit.setText('')		
		self.relationsEdit.setText('')
		self.qeditor.setPlainText('')
		self.aeditor.setPlainText('')
		self.setProperties()
		GUI.showStatus()		

	def setProperties(self):
		ss = open('properties').read().split('\n')
		#print (ss)
		for row in ss:
			sss = row.split(':')
			if sss[0] == 'courseName':
				course.courseName = sss[1]
			elif sss[0] == 'subject':
				course.subject = sss[1]
			elif sss[0] == 'syllabusyear':
				course.syllabusYear = sss[1]
			elif sss[0] == 'semester':
				course.semester = sss[1]
			elif sss[0] == 'degree':
				course.degree = sss[1]
			elif sss[0] == 'language':
				course.language = sss[1]
			elif sss[0] == 'qpModel':
				course.qpModel = sss[1]
			elif sss[0] == 'numModules':
				course.numModules = eval(sss[1])
				m = sss[2:]
				for k in range(course.numModules):
					course.moduleMarks[k] = eval(m[k])	

	def showStatus(self):
		s = course.courseCode + ' | '+course.courseName + ' | ' +\
		course.syllabusYear + ' | ' + course.subject + ' | ' + course.semester + ' semester | Model :'+ course.qpModel
		self.courseInfoLabel.setText(s)
		
app = QApplication(sys.argv)   # Main program
course = Course()
qp = Qpgen()
GUI = MainWindow()
app.exec_()		# main loop

